@@ -4,6 +4,7 @@ from typing import Any, Dict, List, Optional
 
 import boto3
 from botocore.exceptions import ClientError
+from click import ClickException
 
 from anyscale.aws_iam_policies import (
     AMAZON_ECR_READONLY_ACCESS_POLICY_NAME,
@@ -72,6 +73,7 @@ AWS_RESOURCE_DICT: Dict[str, CloudAnalyticsEventCloudResource] = {
     "S3 bucket": CloudAnalyticsEventCloudResource.AWS_S3_BUCKET,
     "EFS": CloudAnalyticsEventCloudResource.AWS_EFS,
     "CloudFormation stack": CloudAnalyticsEventCloudResource.AWS_CLOUDFORMATION,
+    "MemoryDB cluster": CloudAnalyticsEventCloudResource.AWS_MEMORYDB,
 }
 
 
@@ -838,3 +840,105 @@ def verify_aws_cloudformation_stack(
         f"CloudFormation stack {cloud_resource.aws_cloudformation_stack_id} verification succeeded."
     )
     return True
+
+
+def verify_aws_memorydb_cluster(  # noqa: PLR0911, PLR0912
+    cloud_resource: CreateCloudResource,
+    boto3_session: boto3.Session,
+    logger: CloudSetupLogger,
+    strict: bool = False,  # strict is currently unused # noqa: ARG001
+) -> bool:
+    """Verify that the MemoryDB cluster exists and is in the available state."""
+    logger.info("Verifying MemoryDB ...")
+    if not cloud_resource.memorydb_cluster_config:
+        logger.log_resource_error(
+            CloudAnalyticsEventCloudResource.AWS_MEMORYDB,
+            CloudSetupError.MISSING_CLOUD_RESOURCE_ID,
+        )
+        logger.error("Missing MemoryDB cluster id.")
+        return False
+
+    client = boto3_session.client("memorydb")
+    try:
+        response = client.describe_clusters(
+            ClusterName=cloud_resource.memorydb_cluster_config.id.split("/")[-1],
+            ShowShardDetails=True,
+        )
+        if not response.get("Clusters"):
+            log_resource_not_found_error(
+                "MemoryDB cluster", cloud_resource.memorydb_cluster_config.id, logger
+            )
+            return False
+
+        # verify that the subnet group has the same security group as the cloud
+        security_group_id = response["Clusters"][0]["SecurityGroups"][0][
+            "SecurityGroupId"
+        ]
+        if security_group_id != cloud_resource.aws_security_groups[0]:
+            logger.warning(
+                f"MemoryDB cluster {cloud_resource.memorydb_cluster_config.id} has security group {security_group_id} that is not the same as the cloud's security group {cloud_resource.aws_security_groups[0]}."
+            )
+            if strict:
+                return False
+
+        # verify that the cluster is in the cloud's VPC
+        subnet_group_response = client.describe_subnet_groups(
+            SubnetGroupName=response["Clusters"][0]["SubnetGroupName"]
+        )
+        if (
+            subnet_group_response["SubnetGroups"][0]["VpcId"]
+            != cloud_resource.aws_vpc_id
+        ):
+            logger.warning(
+                f"MemoryDB cluster {cloud_resource.memorydb_cluster_config.id} is not in the same VPC as the cloud."
+            )
+            if strict:
+                return False
+
+        # verify that the subnet group has the subset of subnets that the cloud has
+        subnet_ids = [
+            subnet.subnet_id
+            for subnet in cloud_resource.aws_subnet_ids_with_availability_zones
+        ]
+        for subnet in subnet_group_response["SubnetGroups"][0]["Subnets"]:
+            if subnet["Identifier"] not in subnet_ids:
+                logger.warning(
+                    f"MemoryDB cluster {cloud_resource.memorydb_cluster_config.id} has subnet {subnet['Identifier']} that is not one of the subnets in the cloud."
+                )
+                if strict:
+                    return False
+
+        # verify that the cluster has parameter group with the maxmemory-policy set to allkeys-lru
+        parameter_response = client.describe_parameters(
+            ParameterGroupName=response["Clusters"][0]["ParameterGroupName"]
+        )
+        for param in parameter_response["Parameters"]:
+            if param["Name"] == "maxmemory-policy" and param["Value"] != "allkeys-lru":
+                logger.warning(
+                    f"MemoryDB cluster {cloud_resource.memorydb_cluster_config.id} should have parameter group with maxmemory-policy set to allkeys-lru instead of {param['Value']}."
+                )
+                if strict:
+                    return False
+
+        # verify TLS is disabled
+        if response["Clusters"][0]["TLSEnabled"]:
+            logger.error(
+                f"MemoryDB cluster {cloud_resource.memorydb_cluster_config.id} has TLS enabled which is not supported by ray. Please disable TLS for this cluster."
+            )
+            return False
+
+        # verify that each shard in the cluster has at least 2 nodes for high availability
+        for shard in response["Clusters"][0]["Shards"]:
+            if len(shard["Nodes"]) < 2:
+                logger.error(
+                    f"MemoryDB cluster {cloud_resource.memorydb_cluster_config.id} has shard {shard['Name']} with less than 2 nodes. This is not enough for high availability. Please make sure each shard has at least 2 nodes."
+                )
+                return False
+
+    except ClientError as e:
+        logger.log_resource_exception(CloudAnalyticsEventCloudResource.AWS_MEMORYDB, e)
+        raise ClickException(
+            f"Failed to verify MemoryDB cluster {cloud_resource.memorydb_cluster_config.id}.\nError: {e}"
+        )
+
+    return True
