@@ -114,13 +114,6 @@ def cloud_set_default(
     default=False,
     help="Let anyscale create all the resources.",
 )
-@click.option(
-    "--use-legacy-stack",
-    is_flag=True,
-    default=False,
-    hidden=True,
-    help="Whether to use the legacy stack.",
-)
 @click.option(
     "--enable-head-node-fault-tolerance",
     is_flag=True,
@@ -134,22 +127,16 @@ def setup_cloud(  # noqa: PLR0913
     project_id: str,
     functional_verify: Optional[str],
     anyscale_managed: bool,  # noqa: ARG001
-    use_legacy_stack: bool,
     enable_head_node_fault_tolerance: bool,
 ) -> None:
     # TODO (congding): remove `anyscale_managed` in the future, now keeping it for compatibility
-    cluster_management_stack_version = (
-        ClusterManagementStackVersions.V1
-        if use_legacy_stack
-        else ClusterManagementStackVersions.V2
-    )
     if provider == "aws":
         CloudController().setup_managed_cloud(
             provider=provider,
             region=region,
             name=name,
             functional_verify=functional_verify,
-            cluster_management_stack_version=cluster_management_stack_version,
+            cluster_management_stack_version=ClusterManagementStackVersions.V2,
             enable_head_node_fault_tolerance=enable_head_node_fault_tolerance,
         )
     elif provider == "gcp":
@@ -161,7 +148,7 @@ def setup_cloud(  # noqa: PLR0913
             name=name,
             project_id=project_id,
             functional_verify=functional_verify,
-            cluster_management_stack_version=cluster_management_stack_version,
+            cluster_management_stack_version=ClusterManagementStackVersions.V2,
             enable_head_node_fault_tolerance=enable_head_node_fault_tolerance,
         )
 
@@ -192,7 +179,9 @@ def cloud_config_group() -> None:
     pass
 
 
-@cloud_config_group.command("update", help="Update the configuration for a cloud.")
+@cloud_cli.command(
+    name="update", help=("Update managed cloud."),
+)
 @click.argument("cloud-name", required=False)
 @click.option(
     "--cloud-id",
@@ -202,25 +191,32 @@ def cloud_config_group() -> None:
 )
 @click.option("--name", "-n", help="Update configuration of cloud by name.", type=str)
 @click.option(
-    "--max-stopped-instances",
-    help="Maximum number of stopped instances permitted in the shared instance pool.",
-    required=True,
+    "--yes", "-y", is_flag=True, default=False, help="Skip asking for confirmation."
 )
-def cloud_config_update(
+@click.option(
+    "--functional-verify",
+    help="Verify the cloud is functional. This will check that the cloud can launch workspace/service.",
+    required=False,
+    is_flag=False,
+    flag_value="workspace",
+)
+def cloud_update(
     cloud_name: Optional[str],
     name: Optional[str],
     cloud_id: Optional[str],
-    max_stopped_instances: int,
+    functional_verify: Optional[str],
+    yes: bool,
 ) -> None:
     if cloud_name and name and cloud_name != name:
         raise click.ClickException(
             "The positional argument CLOUD_NAME and the keyword argument --name "
             "were both provided. Please only provide one of these two arguments."
         )
-    CloudController().update_cloud_config(
+    CloudController().update_managed_cloud(
         cloud_name=cloud_name or name,
         cloud_id=cloud_id,
-        max_stopped_instances=max_stopped_instances,
+        functional_verify=functional_verify,
+        yes=yes,
     )
 
 
@@ -302,6 +298,9 @@ def cloud_config_get(
 @click.option(
     "--s3-bucket-id", help="S3 bucket ID.", required=False, type=str,
 )
+@click.option(
+    "--memorydb-cluster-id", help="Memorydb cluster ID", required=False, type=str,
+)
 @click.option(
     "--project-id",
     help="Globally Unique project ID for GCP clouds (e.g., my-project-abc123)",
@@ -360,14 +359,13 @@ def cloud_config_get(
     type=str,
 )
 @click.option(
-    "--private-network", help="Use private network.", is_flag=True, default=False,
+    "--memorystore-instance-name",
+    help="Memorystore instance name for GCP clouds",
+    required=False,
+    type=str,
 )
 @click.option(
-    "--use-legacy-stack",
-    is_flag=True,
-    default=False,
-    hidden=True,
-    help="Whether to use the legacy stack.",
+    "--private-network", help="Use private network.", is_flag=True, default=False,
 )
 @click.option(
     "--functional-verify",
@@ -390,6 +388,7 @@ def register_cloud(  # noqa: PLR0913
     instance_iam_role_id: str,
     security_group_ids: str,
     s3_bucket_id: str,
+    memorydb_cluster_id: str,
     project_id: str,
     vpc_name: str,
     subnet_names: str,
@@ -400,13 +399,15 @@ def register_cloud(  # noqa: PLR0913
     provider_name: str,
     firewall_policy_names: str,
     cloud_storage_bucket_name: str,
+    memorystore_instance_name: str,
     functional_verify: Optional[str],
     private_network: bool,
-    use_legacy_stack: bool,
     yes: bool,
 ) -> None:
     missing_args: List[str] = []
     if provider == "aws":
+        # Check for missing required arguments for AWS clouds
+        # The only optional argument is `memorydb_cluster_id`
         for resource in [
             (vpc_id, "--vpc-id"),
             (subnet_ids, "--subnet-ids"),
@@ -432,15 +433,16 @@ def register_cloud(  # noqa: PLR0913
             instance_iam_role_id=instance_iam_role_id,
             security_group_ids=security_group_ids.split(","),
             s3_bucket_id=s3_bucket_id,
+            memorydb_cluster_id=memorydb_cluster_id,
             functional_verify=functional_verify,
             private_network=private_network,
-            cluster_management_stack_version=ClusterManagementStackVersions.V1
-            if use_legacy_stack
-            else ClusterManagementStackVersions.V2,
+            cluster_management_stack_version=ClusterManagementStackVersions.V2,
             yes=yes,
         )
     elif provider == "gcp":
         # Keep the parameter naming ({resource}_name or {resource}_id) consistent with GCP to reduce confusion for customers
+        # Check if all required parameters are provided
+        # Only memorystore_instance_name is optional for GCP clouds
         for resource in [
             (project_id, "--project-id"),
             (vpc_name, "--vpc-name"),
@@ -476,6 +478,17 @@ def register_cloud(  # noqa: PLR0913
                 "Please provide a valid, fully qualified provider name. Example: projects/<project number>/locations/global/workloadIdentityPools/<pool name>/providers/<provider id>"
             )
 
+        if (
+            memorystore_instance_name is not None
+            and re.search(
+                "projects/.+/locations/.+/instances/.+", memorystore_instance_name
+            )
+            is None
+        ):
+            raise click.ClickException(
+                "Please provide a valid memorystore instance name. Example: projects/<project number>/locations/<location>/instances/<instance id>"
+            )
+
         CloudController().register_gcp_cloud(
             region=region,
             name=name,
@@ -490,11 +503,10 @@ def register_cloud(  # noqa: PLR0913
             provider_id=provider_name,
             firewall_policy_names=firewall_policy_names.split(","),
             cloud_storage_bucket_name=cloud_storage_bucket_name,
+            memorystore_instance_name=memorystore_instance_name,
             functional_verify=functional_verify,
             private_network=private_network,
-            cluster_management_stack_version=ClusterManagementStackVersions.V1
-            if use_legacy_stack
-            else ClusterManagementStackVersions.V2,
+            cluster_management_stack_version=ClusterManagementStackVersions.V2,
             yes=yes,
         )
     else:
