@@ -1,5 +1,4 @@
-import sys
-from typing import List, Optional
+from typing import Optional
 
 import click
 from typing_extensions import Literal
@@ -19,44 +18,9 @@ def service_cli() -> None:
     pass
 
 
-@service_cli.command(name="deploy", help="Deploy a service to Anyscale.")
-@click.argument("service-config-file", required=True)
-@click.option("--name", "-n", required=False, default=None, help="Name of service.")
-@click.option(
-    "--description", required=False, default=None, help="Description of service."
-)
-@click.option(
-    "--healthcheck-url", required=False, help="Healthcheck URL for Service.",
-)
-@click.argument(
-    "entrypoint", required=False, nargs=-1,
-)
-def deploy(
-    service_config_file: str,
-    entrypoint: List[str],
-    name: Optional[str],
-    description: Optional[str],
-    healthcheck_url: Optional[str],
-) -> None:
-    # TODO[Bruce]: Remove update flag once fully test apply function.
-    """Deploy a service (create or update).
-
-    This is *only* supported for v1 services.
-    """
-    service_controller = ServiceController()
-    service_controller.deploy(
-        service_config_file,
-        name=name,
-        description=description,
-        healthcheck_url=healthcheck_url,
-        entrypoint=entrypoint,
-        is_entrypoint_cmd="--" in sys.argv,
-    )
-
-
 @service_cli.command(
     name="rollout",
-    help="Rollout a Service v2 to Anyscale. Please contact support for details.",
+    help="Rollout a v2 service to Anyscale. Please contact support for details.",
 )
 @click.option(
     "-f",
@@ -71,7 +35,7 @@ def deploy(
     required=False,
     default=None,
     type=int,
-    help="The percentage of traffic going to the new version of Service that you are deploying",
+    help="The percentage of traffic to send to the canary version of the service.",
 )
 @click.option(
     "--rollout-strategy",
@@ -80,17 +44,34 @@ def deploy(
     type=click.Choice(["ROLLOUT", "IN_PLACE"], case_sensitive=False),
     help="Strategy for rollout.",
 )
+@click.option(
+    "-i",
+    "--in-place",
+    "in_place",
+    is_flag=True,
+    show_default=True,
+    default=False,
+    help="Alias for `--rollout-strategy=IN_PLACE`.",
+)
 def rollout(
     service_config_file: str,
     name: Optional[str],
     version: Optional[str],
     canary_percent: Optional[int],
     rollout_strategy: Optional[Literal["ROLLOUT", "IN_PLACE"]],
+    in_place: bool,
 ) -> None:
     """Start or update a service rollout to a new version.
 
     This is *only* supported for v2 services.
     """
+    if in_place:
+        if rollout_strategy is not None:
+            raise click.ClickException(
+                "Only one of `--in-place/-i` and `--rollout-strategy` can be provided."
+            )
+        rollout_strategy = "IN_PLACE"
+
     service_controller = ServiceController()
     service_controller.rollout(
         service_config_file,
