@@ -11,12 +11,12 @@ class OptionPromptNull(click.Option):
 
     def __init__(self, *args, **kwargs):
         self.default_option = kwargs.pop("default_option", None)
-        super(OptionPromptNull, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
 
     def get_default(self, ctx, **kwargs):
         if not hasattr(self, self._value_key):
             if self.default_option is None:
-                default = super(OptionPromptNull, self).get_default(ctx, **kwargs)
+                default = super().get_default(ctx, **kwargs)
             else:
                 arg = ctx.params.get(self.default_option)
                 if arg is not None:
@@ -41,7 +41,7 @@ class NotRequiredIf(click.Option):
             + " NOTE: This argument is mutually exclusive with %s"
             % self.not_required_if
         ).strip()
-        super(NotRequiredIf, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
 
     def handle_parse_result(self, ctx, opts, args):
         we_are_present = self.name in opts
@@ -57,4 +57,4 @@ class NotRequiredIf(click.Option):
                 self.prompt = None
                 self.default = None
 
-        return super(NotRequiredIf, self).handle_parse_result(ctx, opts, args)
+        return super().handle_parse_result(ctx, opts, args)
