@@ -89,6 +89,7 @@ class PrepareClusterBlock:
         cluster_env_revision: Optional[int],
         ray: Any,
         log_output: bool = True,
+        cluster_compute_version: Optional[int] = None,
     ):
         """
         Class to prepare a cluster (create and start if required) so it can be
@@ -101,6 +102,7 @@ class PrepareClusterBlock:
         self.allow_public_internet_traffic = allow_public_internet_traffic
         self.needs_update = needs_update
         self.cluster_compute_name = cluster_compute_name
+        self.cluster_compute_version = cluster_compute_version
         self.cluster_compute_dict = cluster_compute_dict
         self.cloud_name = cloud_name
         self.build_pr = build_pr
@@ -191,6 +193,7 @@ class PrepareClusterBlock:
                     self.cluster_compute_name,
                     self.cluster_compute_dict,
                     self.cloud_name,
+                    self.cluster_compute_version,
                 )
             else:
                 compute_template_id = existing_terminated_cluster.compute_template_id
@@ -772,6 +775,7 @@ class PrepareClusterBlock:
         cluster_compute_name: Optional[str],
         cluster_compute_dict: Optional[CLUSTER_COMPUTE_DICT_TYPE],
         cloud_name: Optional[str],
+        cluster_compute_version: Optional[int] = None,
     ) -> str:
         """Returns the compute template ids of the cluster to be created.
         By default we return the default cluster compute, unless the user overrides.
@@ -779,7 +783,7 @@ class PrepareClusterBlock:
         # get cluster compute template
         if cluster_compute_name:
             compute_template_id = self._get_cluster_compute_id_from_name(
-                project_id, cluster_compute_name
+                cluster_compute_name, cluster_compute_version
             )
         else:
             # If the user specifies _cluster_compute_dict use it, otherwise
@@ -802,7 +806,9 @@ class PrepareClusterBlock:
         return compute_template_id
 
     def _get_cluster_compute_id_from_name(
-        self, project_id: str, cluster_compute_name: str  # noqa: ARG002
+        self,
+        cluster_compute_name: str,
+        cluster_compute_version: Optional[int] = None,  # noqa: ARG002
     ) -> str:
         """Gets the cluster compute id given a cluster name."""
         cluster_computes = self.api_client.search_compute_templates_api_v2_compute_templates_search_post(
@@ -810,6 +816,7 @@ class PrepareClusterBlock:
                 orgwide=True,
                 name={"equals": cluster_compute_name},
                 include_anonymous=True,
+                version=cluster_compute_version,
             )
         ).results
         if len(cluster_computes) == 0:
@@ -891,6 +898,7 @@ class PrepareClusterBlock:
                 self.cluster_compute_name,
                 self.cluster_compute_dict,
                 self.cloud_name,
+                self.cluster_compute_version,
             )
             if not self._is_equal_cluster_compute(
                 overriding_compute, running_cluster.compute_template_id
@@ -950,6 +958,7 @@ def create_prepare_cluster_block(  # noqa: PLR0913
     cluster_env_revision: Optional[int],
     ray: Any,
     log_output: bool = True,
+    cluster_compute_version: Optional[int] = None,
 ):
     """
     Function to get PrepareClusterBlock object. The PrepareClusterBlock object
@@ -973,6 +982,7 @@ def create_prepare_cluster_block(  # noqa: PLR0913
         cluster_env_revision=cluster_env_revision,
         ray=ray,
         log_output=log_output,
+        cluster_compute_version=cluster_compute_version,
     )
     prepare_cluster_block.prepare()
     return prepare_cluster_block
