@@ -241,7 +241,9 @@ class ComputeConfigController(BaseController):
             compute_config_name = compute_config.name
         else:
             compute_config = get_cluster_compute_from_name(
-                compute_config_entity.name, self.api_client
+                compute_config_entity.name,
+                self.api_client,
+                compute_config_entity.version,
             )
             compute_config_id = compute_config.id
             compute_config_name = compute_config.name
@@ -317,7 +319,10 @@ class ComputeConfigController(BaseController):
         print(f"Cluster computes:\n{table}")
 
     def get(
-        self, cluster_compute_name: Optional[str], cluster_compute_id: Optional[str],
+        self,
+        cluster_compute_name: Optional[str],
+        cluster_compute_id: Optional[str],
+        cluster_compute_version: Optional[int] = None,
     ) -> None:
         if (
             int(cluster_compute_name is not None) + int(cluster_compute_id is not None)
@@ -328,7 +333,7 @@ class ComputeConfigController(BaseController):
             )
         if cluster_compute_name:
             cluster_compute_id = get_cluster_compute_from_name(
-                cluster_compute_name, self.api_client
+                cluster_compute_name, self.api_client, cluster_compute_version
             ).id
         compute_config: ClusterComputeConfig = self.anyscale_api_client.get_cluster_compute(
             cluster_compute_id
