@@ -10,18 +10,13 @@ import tabulate
 
 from anyscale.cli_logger import BlockLogger
 from anyscale.cloud import get_cloud_json_from_id
-from anyscale.cluster_config import get_cluster_config
 from anyscale.controllers.base_controller import BaseController
 from anyscale.formatters import clouds_formatter
 from anyscale.project import (
-    get_proj_name_from_id,
     get_project_id,
-    get_project_session,
-    get_project_sessions,
     load_project_or_throw,
 )
 from anyscale.util import get_endpoint, humanize_timestamp
-from anyscale.utils.imports.all import try_import_ray
 
 
 class ListController(BaseController):
@@ -227,68 +222,6 @@ class ListController(BaseController):
 
             return f"{active_project_title}\n{session_output_string}"
 
-    # TODO (jbai): This requires additional refactoring to decouple network fetching from formatting logic
-    def list_ips(
-        self, session_name: Optional[str], json_format: bool, all_sessions: bool
-    ) -> str:
-        project_definition = load_project_or_throw()
-        project_id = get_project_id(project_definition.root)
-
-        if all_sessions:
-            sessions = get_project_sessions(project_id, session_name, self.api_client)
-            sessions = [session.name for session in sessions]
-        else:
-            sessions = [
-                get_project_session(project_id, session_name, self.api_client).name
-            ]
-
-        output_json = []
-        output_table = []
-        for name in sessions:
-            head_ip, worker_ips = self._get_head_and_worker_ips(session_name=name)
-            if json_format:
-                session = get_project_session(project_id, name, self.api_client)
-                status = self.get_session_status_and_commands(session=session)[0]
-                session_record = {"name": session.name, "status": status}
-                project_record = {
-                    "name": get_proj_name_from_id(project_id, self.api_client)
-                }
-                cloud_record = get_cloud_json_from_id(session.cloud_id, self.api_client)
-                output_json.append(
-                    {
-                        "session": session_record,
-                        "project": project_record,
-                        "cloud": cloud_record,
-                        "head-ip": head_ip,
-                        "worker-ips": worker_ips,
-                    }
-                )
-            else:
-                output_table.append([name, head_ip, "head"])
-                for worker_ip in worker_ips:
-                    output_table.append([name, worker_ip, "worker"])
-
-        return (
-            json.dumps(output_json)
-            if json_format
-            else tabulate.tabulate(
-                output_table,
-                headers=["SESSION", "IP ADDRESS", "NODE TYPE"],
-                tablefmt="plain",
-            )
-        )
-
-    # TODO (jbai): Reorganize to other files
-    def _get_head_and_worker_ips(self, session_name: str) -> Tuple[str, List[str]]:
-        try_import_ray()
-        from ray.autoscaler.sdk import get_head_node_ip, get_worker_node_ips
-
-        cluster_config = get_cluster_config(session_name, self.api_client)
-        head_ip = get_head_node_ip(cluster_config)
-        worker_ips = get_worker_node_ips(cluster_config)
-
-        return head_ip, worker_ips
-
     # Consolidate this once this https://github.com/anyscale/product/pull/497 gets merged.
     def session_list_json(self, sessions: List[Any], project_id: str) -> str:
         output = []
