@@ -1,9 +1,11 @@
 import asyncio
+from collections import OrderedDict
 from datetime import timedelta
 import errno
+import json
 import os
 import tempfile
-from typing import List, Optional
+from typing import Any, List, Optional, Tuple
 
 import aiohttp
 import click
@@ -36,6 +38,12 @@ DEFAULT_READ_TIMEOUT = 30
 DEFAULT_TTL = 14400
 DEFAULT_PARALLELISM = 10
 
+# Most users should have be able to handle this many open file descriptors,
+# but add ability to override just in case.
+DEFAULT_UNPACK_MAX_FILE_DESCRIPTORS = int(
+    os.environ.get("ANYSCALE_LOGS_UNPACK_MAX_FILE_DESCRIPTORS", "100")
+)
+
 
 class LogsController(BaseController):
     def __init__(
@@ -47,6 +55,9 @@ class LogsController(BaseController):
         super().__init__(initialize_auth_api_client=initialize_auth_api_client)
         self.log = log
         self.console = Console()
+        # Track if we've already sent the unpack log message, so that we don't
+        # spam the CLI output when users have had multiple sessions.
+        self._unpack_log_sent = False
 
     def render_logs(
         self, log_group: LogGroup, parallelism: int, read_timeout: timedelta, tail: int
@@ -76,13 +87,13 @@ class LogsController(BaseController):
                     "The latest 24 hours of logs are not guaranteed if the cluster is still running. "
                     "If so, please view logs via the Anyscale UI or Ray dashboard."
                 )
-        chunks = self._list_log_chunks(
+        chunks, bearer_token = self._list_log_chunks(
             log_filter=filter,
             page_size=page_size,
             ttl_seconds=ttl_seconds,
             timeout=timeout,
         )
-        log_group = self._group_log_chunk_list(chunks)
+        log_group = self._group_log_chunk_list(chunks, bearer_token)
         return log_group
 
     def download_logs(  # noqa: PLR0913
@@ -98,6 +109,7 @@ class LogsController(BaseController):
         parallelism: int = DEFAULT_PARALLELISM,
         download_dir: Optional[str] = None,
         write_to_stdout: bool = False,
+        unpack: bool = True,
     ) -> None:
         log_group = self.get_log_group(filter, page_size, ttl_seconds, timeout)
         self.console.log(
@@ -111,6 +123,7 @@ class LogsController(BaseController):
             log_group=log_group,
             show_progress_bar=True,
             write_to_stdout=write_to_stdout,
+            unpack=unpack,
         )
 
     # TODO (shomilj): Refactor this method. It's too nested.
@@ -123,6 +136,7 @@ class LogsController(BaseController):
         write_to_stdout: bool = False,
         tail: int = -1,
         show_progress_bar: bool = False,
+        unpack: bool = True,
     ):
         if len(log_group.get_chunks()) == 0:
             return
@@ -135,6 +149,7 @@ class LogsController(BaseController):
                     self._download_files(
                         base_folder=tmp_dir,
                         log_chunks=log_group.get_chunks(),
+                        bearer_token=log_group.bearer_token,
                         parallelism=parallelism,
                         read_timeout=read_timeout,
                         show_progress_bar=show_progress_bar,
@@ -149,7 +164,7 @@ class LogsController(BaseController):
                         lines_read = 0
                         for chunk in chunks:
                             with open(
-                                os.path.join(tmp_dir, chunk.chunk_name)
+                                os.path.join(tmp_dir, chunk.chunk_name), errors="ignore"
                             ) as source:
                                 if tail > 0:
                                     # Tail is enabled, so read log lines in reverse.
@@ -191,6 +206,9 @@ class LogsController(BaseController):
                                     dest.write("\n")
                                 chunks_written += 1
 
+                        if unpack:
+                            self._unpack_structured_log(real_path)
+
                         if chunks_written == 0:
                             os.remove(real_path)
 
@@ -210,10 +228,12 @@ class LogsController(BaseController):
                     "ulimit -n $(($(ulimit -n) * 2))"
                 )
 
-    def _group_log_chunk_list(self, chunks: List[LogFileChunk]) -> LogGroup:
+    def _group_log_chunk_list(
+        self, chunks: List[LogFileChunk], bearer_token: Optional[str] = None
+    ) -> LogGroup:
         # This has to happen locally because it happens after we retrieve all file metadata through the paginated
         # backend API for listing S3/GCS buckets.
-        group = LogGroup()
+        group = LogGroup(bearer_token)
         for chunk in chunks:
             group.insert_chunk(chunk=chunk)
         return group
@@ -224,9 +244,10 @@ class LogsController(BaseController):
         page_size: Optional[int],
         ttl_seconds: Optional[int],
         timeout: timedelta,
-    ) -> List[LogFileChunk]:
+    ) -> Tuple[List[LogFileChunk], Optional[str]]:
         next_page_token: Optional[str] = None
         all_log_chunks: List[LogFileChunk] = []
+        bearer_token = None
 
         with self.console.status("Scanning available logs...") as status:
             while True:
@@ -236,11 +257,13 @@ class LogsController(BaseController):
                         next_page_token=next_page_token,
                         page_size=page_size,
                         ttl_seconds=ttl_seconds,
+                        use_bearer_token=True,
                     ),
                 )
                 result: LogDownloadResult = self.api_client.get_log_files_api_v2_logs_get_log_files_post(
                     log_download_request=request, _request_timeout=timeout
                 ).result
+                bearer_token = result.bearer_token
                 all_log_chunks.extend(result.log_chunks)
                 if status:
                     status.update(
@@ -253,7 +276,7 @@ class LogsController(BaseController):
                     break
                 next_page_token = result.next_page_token
 
-        return all_log_chunks
+        return all_log_chunks, bearer_token
 
     async def _download_file(  # noqa: PLR0913
         self,
@@ -264,6 +287,7 @@ class LogsController(BaseController):
         size: int,  # noqa: ARG002
         session: aiohttp.ClientSession,
         read_timeout: timedelta,
+        bearer_token: Optional[str] = None,
     ) -> None:
         async with sem:
             download_dir = os.path.dirname(file_name)
@@ -273,7 +297,10 @@ class LogsController(BaseController):
             timeout = aiohttp.ClientTimeout(
                 total=None, sock_connect=30, sock_read=read_timeout.seconds
             )
-            async with session.get(url, timeout=timeout) as response:
+            headers = (
+                {"Authorization": f"Bearer {bearer_token}"} if bearer_token else {}
+            )
+            async with session.get(url, timeout=timeout, headers=headers) as response:
                 if response.status == 200:
                     with open(file_name, "wb") as fhand:
                         async for chunk in response.content.iter_chunked(1024):
@@ -290,6 +317,7 @@ class LogsController(BaseController):
         parallelism: int,
         read_timeout: timedelta,
         show_progress_bar: bool = False,
+        bearer_token: Optional[str] = None,
     ) -> List[str]:
         sem = asyncio.Semaphore(parallelism)
         downloads = []
@@ -309,6 +337,7 @@ class LogsController(BaseController):
                             log_chunk.size,
                             session,
                             read_timeout,
+                            bearer_token=bearer_token,
                         )
                     )
                 )
@@ -325,3 +354,88 @@ class LogsController(BaseController):
                 await asyncio.gather(*downloads)
 
         return paths
+
+    def _unpack_structured_log(self, log_path: str):
+        directory = os.path.dirname(log_path)
+        fname = os.path.basename(log_path)
+        if fname != "combined-worker.log":
+            return
+        if not self._unpack_log_sent:
+            self.console.log("Unpacking `combined-worker.log` into separate files.")
+            self._unpack_log_sent = True
+        error_count = 0
+        seen = set()
+        with _FileDescriptorCache(DEFAULT_UNPACK_MAX_FILE_DESCRIPTORS) as fds, open(
+            log_path
+        ) as f:
+            for line in f.readlines():
+                try:
+                    j = json.loads(line)
+                    msg = j["message"]
+                    filename = os.path.join(directory, j["filename"])
+
+                    # Track files we've seen. The first time we see a file,
+                    # we should use "wt" to overwrite previous content, for
+                    # example if a user uses the download command multiple
+                    # times. On subsequent times, we use "at" to append to
+                    # existing content, in case the file descriptor has been
+                    # closed in the meantime.
+                    if filename in seen:
+                        mode = "at"
+                    else:
+                        seen.add(filename)
+                        mode = "wt"
+
+                    f = fds.get(filename, mode)
+                    f.write(f"{msg}\n")
+                except KeyboardInterrupt:
+                    raise
+                except OSError as err:
+                    if err.errno == errno.EMFILE:
+                        raise
+                    error_count += 1
+                except Exception:  # noqa: BLE001
+                    # Soft fail on unpacking the combined log. Worst case scenario,
+                    # users can still parse the combined log manually
+                    error_count += 1
+        if error_count:
+            self.log.warning(
+                f"{error_count} error(s) while unpacking {fname}. Check combined-worker.log "
+                "for the full contents of worker logs."
+            )
+
+
+class _FileDescriptorCache:
+    """
+    LRU Cache for file descriptors to use while uncombining worker logs, since
+    opening and closing a file descriptor per line would be slow.
+    """
+
+    def __init__(self, max_entries: int):
+        self.max_entries = max_entries
+        # Recently used entries are near the "end"
+        self.cache: OrderedDict[str, Any] = OrderedDict()
+
+    def __enter__(self) -> "_FileDescriptorCache":
+        return self
+
+    def __exit__(self, exc_type: Any, exc_value: Any, tb: Any):
+        for f in self.cache.values():
+            try:
+                f.close()
+            except Exception:  # noqa: BLE001
+                # There's not much we can do if we fail to close a file descriptor,
+                # just move on to the next one.
+                continue
+
+    def get(self, filename: str, mode: str):
+        if filename in self.cache:
+            # Update position in cache
+            self.cache.move_to_end(filename)
+            return self.cache[filename]
+        if len(self.cache) >= self.max_entries:
+            # Pop and close file descriptor closest to the "front"
+            _, f = self.cache.popitem(last=False)
+            f.close()
+        self.cache[filename] = open(filename, mode)  # noqa: SIM115
+        return self.cache[filename]
