@@ -62,7 +62,7 @@ class ProjectController(BaseController):
             project_id=project_id,
         )
 
-    def create(self, name: str) -> None:
+    def create(self, name: str, parent_cloud_id: Optional[str]) -> None:
         """
         Call API to create a new project given a project name.
         """
@@ -80,12 +80,23 @@ class ProjectController(BaseController):
                 f"Project with name {name} is already created by you at {url}."
             )
         else:
+            if not parent_cloud_id and (
+                self.api_client.check_is_feature_flag_on_api_v2_userinfo_check_is_feature_flag_on_get(
+                    "cloud-isolation-phase-1"
+                ).result.is_on
+                is True
+            ):
+                raise click.ClickException(
+                    "Parent cloud id should be specified when creating a new projet because "
+                    "cloud isolation has been enabled."
+                )
             # Create new project
             project_id = self.api_client.create_project_api_v2_projects_post(
                 write_project={
                     "name": name,
                     "description": "",
                     "initial_cluster_config": None,
+                    "parent_cloud_id": parent_cloud_id,
                 }
             ).result.id
             url = get_endpoint(f"/projects/{project_id}")
