@@ -247,8 +247,21 @@ class ServiceController(BaseController):
             cluster_compute_id=config.compute_config_id,
             cluster_compute=config.compute_config,
             cloud=config.cloud,
+            cluster_compute_version=config.compute_config_version,
         )
 
+        service_config = config.config
+        if service_config:
+            access_config = service_config.get("access", {})
+
+            if (
+                isinstance(access_config, dict)
+                and access_config.get("use_bearer_token", True) is False
+            ):
+                self.log.warning(
+                    "Using bearer token for authorization is disabled: no authorization guarding will be used for this service."
+                )
+
         if not config.ray_serve_config:
             config.ray_serve_config = {}
 
