@@ -665,3 +665,64 @@ def _check_bucket_region(bucket: Bucket, region: str) -> Tuple[bool, str]:
     # Bucket is `multi-region`, so check if the location (`EU`, `ASIA`, `US`)
     # is the 'prefix' of the region.
     return region.upper().startswith(bucket.location), bucket.location
+
+
+def verify_memorystore(
+    factory: GoogleCloudClientFactory,
+    resources: CreateCloudResourceGCP,
+    logger: GCPLogger,
+    strict: bool = False,
+):
+    """Verify that the Google Memorystore exists & raises warnings about improper configurations."""
+    client = factory.redis_v1.CloudRedisClient()
+
+    redis_instance_name = resources.memorystore_instance_config.name
+
+    try:
+        redis_instance = client.get_instance(name=redis_instance_name)
+    except (NotFound, PermissionDenied):
+        logger.log_resource_not_found_error("Memorystore", redis_instance_name)
+        return False
+
+    # Verify memorystore is in the same VPC as the cloud
+    cloud_vpc_name = resources.gcp_vpc_id
+    redis_vpc_name = redis_instance.authorized_network.split("/")[-1]
+    if cloud_vpc_name != redis_vpc_name:
+        logger.internal.log_resource_error(
+            CloudAnalyticsEventCloudResource.GCP_MEMORYSTORE,
+            CloudSetupError.MEMORYSTORE_NOT_CONNECTED_TO_VPC,
+        )
+        logger.internal.error(
+            f"Memorystore is not connected to {cloud_vpc_name}, but to {redis_vpc_name}. "
+            f"This cannot be edited on an existing Memorystore instance. Please recreate the Memorystore and connect it to {cloud_vpc_name}."
+        )
+        return False
+
+    # Verify memorystore is the standard tier
+    if redis_instance.tier.name != "STANDARD_HA":
+        logger.internal.warning(
+            "Memorystore is not in the standard tier. This can result in degraded performance and availability. Note that this cannot be changed on an existing Memorystore instance."
+        )
+        if strict:
+            return False
+
+    # Verify that memorystore instance has the maxmemory-policy set to allkeys-lru
+    if redis_instance.redis_configs.get("maxmemory-policy") != "allkeys-lru":
+        logger.internal.warning(
+            "Memorystore does not have the maxmemory-policy set to allkeys-lru. This can result in degraded performance and availability."
+        )
+        if strict:
+            return False
+
+    # Verify that the memorystore instance has the read replica mode turned on
+    if redis_instance.read_replicas_mode.name != "READ_REPLICAS_ENABLED":
+        logger.internal.error(
+            "Memorystore does not have the read replica mode turned on. This is required for ray head node HA."
+        )
+        logger.internal.log_resource_error(
+            CloudAnalyticsEventCloudResource.GCP_MEMORYSTORE,
+            CloudSetupError.MEMORYSTORE_READ_REPLICAS_DISABLED,
+        )
+        return False
+
+    return True
