@@ -4,7 +4,7 @@ from typing import Any, Dict, List, Optional, Union
 from urllib.parse import urlparse
 
 import click
-from pydantic import BaseModel, Extra, Field, root_validator
+from pydantic import BaseModel, Extra, Field, root_validator, validator
 import yaml
 
 from anyscale.cli_logger import LogsLogger
@@ -292,7 +292,7 @@ class BaseHAJobConfig(BaseModel):
     )
     compute_config: Optional[Union[str, Dict[str, Any]]] = Field(
         None,
-        description="The name of the compute configuration that you want to use. This will specify the resources required for your job."
+        description="The name of the compute configuration and version that you want to use in format `my_compute_config:1`. This will specify the resources required for your job."
         "This field also accepts a one-off config as a dictionary.",
     )
     cloud: Optional[str] = Field(
@@ -303,6 +303,19 @@ class BaseHAJobConfig(BaseModel):
         5,
         description="The number of retries this job will attempt on failure. Set to None to set infinite retries",
     )
+    compute_config_version: Optional[int] = Field(
+        None,
+        description="The compute config version filter that you want to use. Only applicable if the compute_config was specified by name. "
+        "Versions are positive integers. "
+        "Deprecated behavior: Setting version to None is equivalent to setting version to '1'.",
+    )
+
+    @validator("compute_config_version")
+    def optional_positive_int(cls, v):
+        assert (
+            v is None or v >= 1
+        ), "compute_config_version must be a positive integer or None"
+        return v
 
     class Config:
         extra = Extra.forbid
@@ -378,10 +391,11 @@ class BaseHAJobConfig(BaseModel):
 
     @root_validator
     def fill_compute_config_id(cls: Any, values: Any) -> Any:
-        compute_config_id, compute_config, cloud = (
+        compute_config_id, compute_config, cloud, compute_config_version = (
             values.get("compute_config_id"),
             values.get("compute_config"),
             values.get("cloud"),
+            values.get("compute_config_version"),
         )
         project_id = values.get("project_id")
         if not project_id:
@@ -393,7 +407,9 @@ class BaseHAJobConfig(BaseModel):
                 "Only one of `compute_config_id`, `compute_config`, or `cloud` can be provided in the config file."
             )
         if compute_config and isinstance(compute_config, str):
-            compute_config_id = get_cluster_compute_from_name(compute_config).id
+            compute_config_id = get_cluster_compute_from_name(
+                compute_config, version=compute_config_version
+            ).id
         elif compute_config and isinstance(compute_config, dict):
             compute_config_id = register_compute_template(compute_config).id
         elif cloud:
