@@ -1,4 +1,6 @@
+# This file is being deprecated and will be migrated to go/infra/config/anyscaled/systemd/config-template/snapshot_util.py
 # NOTE: The logic in this file/module is only run from within an Anyscale workspace
+import asyncio
 import datetime
 import json
 import logging
@@ -10,6 +12,7 @@ import tarfile
 import tempfile
 import time
 from typing import Any, Dict, Optional, Tuple
+from urllib.parse import urlparse
 
 from anyscale.utils.imports.all import try_import_ray
 
@@ -72,6 +75,8 @@ EFS_OBJECTS_DIR = os.environ.get("EFS_OBJECTS_DIR", "/efs/workspaces/shared_obje
 EFS_CREDS_DIR = os.environ.get("EFS_CREDS_DIR", "/efs/generated_credentials")
 RAY_ML_DEV = bool(os.environ.get("RAY_ML_DEV"))
 AUTOGC = bool(os.environ.get("AUTOGC_SNAPSHOTS", True))
+SNAPSHOT_RETENTION_COUNT = max(int(os.environ.get("SNAPSHOT_RETENTION_COUNT", 5)), 1)
+SNAPSHOT_RETENTION_HOURS = max(int(os.environ.get("SNAPSHOT_RETENTION_HOURS", 1)), 1)
 SKIP_VSCODE_DESKTOP_SETUP = bool(os.environ.get("SKIP_VSCODE_DESKTOP_SETUP", False))
 
 # This causes workspaces to use the $CWD as the runtime env working dir instead of
@@ -227,12 +232,9 @@ def unpack_github_dir(github_tree_uri: str, out_dir: str) -> None:
     tmp = tempfile.mktemp(prefix=f"github_{gh_repo}", suffix=".tar.gz")
     subprocess.check_call(
         [
-            "curl",
-            "--fail",
-            "-L",
+            "wget",
+            f"--output-document={tmp}",
             f"https://github.com/{gh_user}/{gh_repo}/tarball/{gh_branch}",
-            "--output",
-            tmp,
         ]
     )
     logger.info(f"Downloaded tarball to {tmp}")
@@ -366,7 +368,7 @@ def report_idle_termination_error(err: str):
 
 
 def gc_snapshots() -> None:
-    """Garbage collect snapshots older than 1 hr.
+    """Garbage collect snapshots older than X hours.
 
     This is safe since when we clone a workspace, the autosave loop will generate a
     new snapshot, so the clone will be decoupled from the original.
@@ -376,11 +378,18 @@ def gc_snapshots() -> None:
     if not os.path.exists(snapshot_dir):
         return
     snapshots = sorted([x for x in os.listdir(snapshot_dir) if x.endswith(".zip")])
-    # Never GC the latest snapshot.
-    snapshots = snapshots[:-1]
-    horizon = datetime.datetime.now(datetime.timezone.utc) - datetime.timedelta(hours=1)
+    # Never GC the latest SNAPSHOT_RETENTION_COUNT snapshot.
+    snapshots = snapshots[:-SNAPSHOT_RETENTION_COUNT]
+    horizon = datetime.datetime.now(datetime.timezone.utc) - datetime.timedelta(
+        hours=SNAPSHOT_RETENTION_HOURS
+    )
+
+    logger.debug(
+        f"Snapshot retention count: {SNAPSHOT_RETENTION_COUNT}, snapshot retention hours: {SNAPSHOT_RETENTION_HOURS}"
+    )
+
     prefix = f"snapshot_{horizon.isoformat()}_"
-    # GC snapshots older than 1 hour.
+    # GC snapshots older than SNAPSHOT_RETENTION_HOURS hour.
     for snapshot in snapshots:
         if snapshot < prefix:
             full_path = os.path.join(snapshot_dir, snapshot)
@@ -408,7 +417,90 @@ def find_latest(ignore_base_snapshot: bool) -> Optional[str]:
     return os.path.join(snapshot_dir, snapshots[-1])
 
 
+def find_base_snapshot_from_job(job_id: str) -> Optional[str]:
+    if not job_id:
+        logger.info("Invalid base snapshot, no job id")
+        return None
+    logger.info(f"Base snapshot from job {job_id}")
+
+    # find service snapshot URI from remote storage
+    from anyscale.authenticate import get_auth_api_client
+
+    auth_api_client = get_auth_api_client(log_output=False)
+    api_client = auth_api_client.api_client
+    job = api_client.get_job_api_v2_decorated_ha_jobs_production_job_id_get(
+        job_id
+    ).result
+    if (
+        job
+        and job.config
+        and job.config.runtime_env
+        and job.config.runtime_env.working_dir
+    ):
+        return job.config.runtime_env.working_dir
+
+    # fallback to find job snapshot in EFS
+    job_efs_snapshot = os.path.join(EFS_JOB_DIR, job_id, "working_dir.zip")
+    if os.path.exists(job_efs_snapshot):
+        return job_efs_snapshot
+
+    return None
+
+
+def find_base_snapshot_from_service(service_id: str) -> Optional[str]:
+    if not service_id:
+        logger.info("Invalid base snapshot, no service id")
+        return None
+    logger.info(f"Base snapshot from service {service_id}")
+
+    # find service snapshot URI from remote storage
+    from anyscale.authenticate import get_auth_api_client
+
+    auth_api_client = get_auth_api_client(log_output=False)
+    api_client = auth_api_client.api_client
+    service = api_client.get_job_api_v2_decorated_ha_jobs_production_job_id_get(
+        service_id
+    ).result
+    if service and service.config and service.config.ray_serve_config:
+        runtime_env = service.config.ray_serve_config.get("runtime_env", {})
+        if "working_dir" in runtime_env:
+            return runtime_env["working_dir"]
+
+    return None
+
+
+def find_base_snapshot_from_workspace(
+    workspace_id: str, iso_time: str
+) -> Optional[str]:
+    if not workspace_id or not iso_time:
+        logger.info("Invalid base snapshot, no workspace id or time")
+        return None
+    logger.info(f"Base snapshot from workspace {workspace_id} and {iso_time}")
+    snapshot_dir = os.path.join(EFS_WORKSPACE_DIR, workspace_id, "snapshots")
+    snapshot_time = f"snapshot_{iso_time}_"
+    if not os.path.exists(snapshot_dir):
+        return None
+    # Find the latest matching snapshot before the time stamp
+    snapshots = sorted(
+        [
+            x
+            for x in os.listdir(snapshot_dir)
+            if x.endswith(".zip") and x < snapshot_time
+        ]
+    )
+    if not snapshots:
+        return None
+    return os.path.join(snapshot_dir, snapshots[-1])
+
+
 def find_base_snapshot() -> Optional[str]:  # noqa: PLR0911
+    """
+        Find the base snapshot for the job/service/workspace used to create current workspace.
+
+        Returns:
+            If exists, return the EFS path or URI of the base snapshot.
+            otherwise, return None.
+    """
     if not BASE_SNAPSHOT:
         return None
     try:
@@ -419,45 +511,98 @@ def find_base_snapshot() -> Optional[str]:  # noqa: PLR0911
 
     # Jobs snapshot
     if "from_job" in base_data:
-        job_id = base_data["from_job"]["job_id"]
-        if not job_id:
-            logger.info("Invalid base snapshot, no job id")
-            return None
-        logger.info(f"Base snapshot from job {job_id}")
-        return os.path.join(EFS_JOB_DIR, job_id, "working_dir.zip")
+        job_id = base_data["from_job"].get("job_id")
+        return find_base_snapshot_from_job(job_id=job_id)
+
+    # Services snapshot
+    if "from_service" in base_data:
+        service_id = base_data["from_service"].get("service_id")
+        return find_base_snapshot_from_service(service_id=service_id)
 
     # Workspace snapshot
     if "from_workspace" in base_data:
-        workspace_id = base_data["from_workspace"]["workspace_id"]
-        iso_time = base_data["from_workspace"]["iso_time"]
-        if not workspace_id or not iso_time:
-            logger.info("Invalid base snapshot, no workspace id or time")
-            return None
-        logger.info(f"Base snapshot from workspace {workspace_id} and {iso_time}")
-        snapshot_dir = os.path.join(EFS_WORKSPACE_DIR, workspace_id, "snapshots")
-        snapshot_time = f"snapshot_{iso_time}_"
-        if not os.path.exists(snapshot_dir):
-            return None
-        # Find the latest matching snapshot before the time stamp
-        snapshots = sorted(
-            [
-                x
-                for x in os.listdir(snapshot_dir)
-                if x.endswith(".zip") and x < snapshot_time
-            ]
-        )
-        if not snapshots:
-            return None
-        return os.path.join(snapshot_dir, snapshots[-1])
+        workspace_id = base_data["from_workspace"].get("workspace_id")
+        iso_time = base_data["from_workspace"].get("iso_time")
+        return find_base_snapshot_from_workspace(workspace_id, iso_time)
 
+    logger.info(
+        "Failed to find base snapshot since it's not from job/service/workspace"
+    )
     return None
 
 
+def _is_uri(uri: str) -> bool:
+    """Check if the given string is a valid URI.
+    Note that this function does not check if the URI is accessable."""
+    parsed_uri = urlparse(uri)
+    return bool(parsed_uri.scheme and parsed_uri.netloc)
+
+
+def _move_files(src_dir: str, dest_dir: str) -> None:
+    """
+    Moves all files in the source directory to the destination directory.
+
+    Exceptions:
+        1. shutil.Error: raised when a general error occurs, such as a file already existing in the destination directory with the same name as a file being moved.
+        2. OSError: raised when a file-related system error occurs, such as an invalid source or destination file path, or a permission error.
+        3. IOError: raised when an input/output error occurs, such as a disk error or a file being locked by another process.
+    """
+    try:
+        # create the destination directory if it doesn't exist
+        if not os.path.exists(dest_dir):
+            os.makedirs(dest_dir)
+
+        # get a list of all files in the source directory
+        files = os.listdir(src_dir)
+
+        # loop through all files and move them to the destination directory
+        for file in files:
+            src_path = os.path.join(src_dir, file)
+            dest_path = os.path.join(dest_dir, file)
+            shutil.move(src_path, dest_path)
+
+    except Exception as e:
+        logger.exception(
+            f"Failed to move files from {src_dir} to {dest_dir} due to {e}"
+        )
+        raise
+
+
+def _download_and_unpack_snapshot_uri(snapshot_uri: str, working_dir: str) -> None:
+    """
+    Download the snapshot from the given URI and unpack it to the working directory.
+    The snapshot will be first downloaded to a temporary directory and then moved to the working directory.
+
+    Args:
+        snapshot_uri: URI pointing to the snapshot.
+        working_dir: Output directory to be polulated with the downloaded contents.
+    """
+
+    try:
+        # use Ray DeveloperAPI to download the snapshot from remote buckets
+        from ray._private.runtime_env.packaging import download_and_unpack_package
+
+        with tempfile.TemporaryDirectory() as temp_dir:
+            unpacked_dir = asyncio.run(
+                download_and_unpack_package(
+                    pkg_uri=snapshot_uri, base_directory=temp_dir
+                )
+            )
+            _move_files(src_dir=unpacked_dir, dest_dir=working_dir)
+
+            logger.info(f"Downloaded snapshot from {snapshot_uri} to {working_dir}")
+    except Exception as e:
+        logger.exception(f"Failed to download snapshot from {snapshot_uri} due to {e}")
+        raise
+
+
 def restore_latest():
     """Command to restore the latest snapshot within an Anyscale workspace.
 
     Can be run via `python -m anyscale.snapshot_util restore`.
     """
+
+    # Step 1: Find the latest snapshot.
     latest = find_latest(ignore_base_snapshot=False)
     logger.info(f"Latest snapshot found was {latest}")
     if not latest:
@@ -465,8 +610,15 @@ def restore_latest():
         if WORKSPACE_TEMPLATE:
             unpack_github_dir(WORKSPACE_TEMPLATE, WORKING_DIR)
         return
+
+    # Step 2: Download/Unpack the snapshot.
+    if _is_uri(latest):
+        _download_and_unpack_snapshot_uri(snapshot_uri=latest, working_dir=WORKING_DIR)
+    else:
+        unpack_snapshot_zip(latest, WORKING_DIR)
+
+    # Step 3: Copy over the config files.
     workspace_dir = os.path.join(EFS_WORKSPACE_DIR, WORKSPACE_ID)
-    unpack_snapshot_zip(latest, WORKING_DIR)
     for config in CONFIGS_TO_SAVE:
         saved = os.path.join(workspace_dir, config)
         if os.path.exists(saved):
@@ -482,7 +634,9 @@ def _split_s3_path(s3_path: str) -> Tuple[str, str]:
 
 
 def checkpoint_job(job_id, runtime_env: Dict[str, Any]) -> Dict[str, Any]:
-    """Checkpoint the runtime environment and working directory of a job.
+    """ Deprecated in v2 stack: we are moving away from EFS and towards remote buckets.
+
+    Checkpoint the runtime environment and working directory of a job.
 
     This function will modify runtime_env to point to the new working
     directory on EFS and return the modified runtime_env.
@@ -506,6 +660,11 @@ def checkpoint_job(job_id, runtime_env: Dict[str, Any]) -> Dict[str, Any]:
             # boto3 client initialized inside clusters already have AWS credentials set
             s3_client = boto3.client("s3")
             s3_client.download_file(bucket, key, working_dir)
+        elif runtime_env["working_dir"].startswith("gs://"):
+            # checkpoint_job is deprecated in v2 stack,
+            # so we don't need to support gs:// uris here.
+            # just return here to avoid breaking the code
+            return runtime_env
         else:
             import urllib.request
 
@@ -734,6 +893,7 @@ def setup_nfs():
             subprocess.check_call(cmd, shell=True)
         except Exception:
             logger.exception(f"Error running {cmd}")
+            raise
 
     # on first load, report an activity to signal healthy start
     report_idle_termination_activity()
@@ -776,6 +936,7 @@ def setup_container(ray_params: Any, is_head: bool):  # noqa: ARG001, PLR0912
             setup_nfs()
         except Exception:
             logger.exception("Failed to setup NFS")
+            raise
         if not WORKSPACE_ID:
             logger.info("No workspace id configured, skipping workspace restore.")
         elif not is_head:
