@@ -33,8 +33,10 @@ import requests
 from requests import Response
 
 from anyscale.authenticate import get_auth_api_client
+from anyscale.aws_iam_policies import ANYSCALE_IAM_POLICIES, AnyscaleIAMPolicy
 from anyscale.cli_logger import BlockLogger, CloudSetupLogger
 from anyscale.client.openapi_client.api.default_api import DefaultApi as ProductApi
+from anyscale.client.openapi_client.models import AWSMemoryDBClusterConfig
 from anyscale.client.openapi_client.models.cloud_analytics_event_cloud_resource import (
     CloudAnalyticsEventCloudResource,
 )
@@ -48,6 +50,7 @@ from anyscale.conf import MINIMUM_RAY_VERSION
 from anyscale.sdk.anyscale_client.api.default_api import DefaultApi as BaseApi
 from anyscale.sdk.anyscale_client.models.compute_template import ComputeTemplate
 import anyscale.shared_anyscale_utils.conf as shared_anyscale_conf
+from anyscale.utils.cloud_utils import CloudSetupError
 
 
 logger = logging.getLogger(__file__)
@@ -207,7 +210,9 @@ def _get_role(role_name: str, region: str) -> Optional[Boto3Resource]:
             raise exc
 
 
-def _get_aws_efs_mount_target_ip(boto3_session: Any, efs_id: str) -> Optional[str]:
+def _get_aws_efs_mount_target_ip(
+    boto3_session: Any, efs_id: str, cloud_setup_logger: CloudSetupLogger
+) -> Optional[str]:
     client = boto3_session.client("efs")
     try:
         mount_targets_response = client.describe_mount_targets(FileSystemId=efs_id)
@@ -218,6 +223,9 @@ def _get_aws_efs_mount_target_ip(boto3_session: Any, efs_id: str) -> Optional[st
     except ClientError as e:
         if e.response["Error"]["Code"] == "FileSystemNotFound":
             return None
+        cloud_setup_logger.log_resource_exception(
+            CloudAnalyticsEventCloudResource.AWS_EFS, e
+        )
         raise e
 
 
@@ -231,7 +239,13 @@ def _get_subnet(
         return subnet  # type: ignore
     except ClientError as e:
         if e.response["Error"]["Code"] == "InvalidSubnetID.NotFound":
-            return None
+            logger.log_resource_error(
+                CloudAnalyticsEventCloudResource.AWS_SUBNET,
+                CloudSetupError.RESOURCE_NOT_FOUND,
+            )
+            raise click.ClickException(
+                f"{subnet_arn} does not exist. Please make sure the subnet arn is correct and the subnet is in the same region as the cloud."
+            )
         else:
             logger.log_resource_exception(
                 CloudAnalyticsEventCloudResource.AWS_SUBNET, e
@@ -239,6 +253,39 @@ def _get_subnet(
         raise e
 
 
+def _get_memorydb_cluster_config(
+    memorydb_cluster_id: str, region: str, logger: CloudSetupLogger
+) -> Optional[AWSMemoryDBClusterConfig]:
+    try:
+        memorydb_client = boto3.client("memorydb", region_name=region)
+        response = memorydb_client.describe_clusters(ClusterName=memorydb_cluster_id)
+
+        if not response.get("Clusters") or not response.get("Clusters")[0]:
+            logger.log_resource_error(
+                CloudAnalyticsEventCloudResource.AWS_MEMORYDB,
+                CloudSetupError.RESOURCE_NOT_FOUND,
+            )
+            raise click.ClickException(
+                f"MemoryDB cluster with id {memorydb_cluster_id} does not exist."
+            )
+        cluster = response["Clusters"][0]
+        endpoint = cluster["ClusterEndpoint"]
+        return AWSMemoryDBClusterConfig(
+            id=cluster["ARN"], endpoint=f"{endpoint['Address']}:{endpoint['Port']}",
+        )
+    except ClientError as e:
+        if e.response["Error"]["Code"] == "ClusterNotFoundFault":
+            logger.log_resource_error(
+                CloudAnalyticsEventCloudResource.AWS_MEMORYDB,
+                CloudSetupError.RESOURCE_NOT_FOUND,
+            )
+        else:
+            logger.log_resource_exception(
+                CloudAnalyticsEventCloudResource.AWS_MEMORYDB, e
+            )
+        raise e
+
+
 def get_available_regions() -> List[str]:
     try:
         client = boto3.client("ec2")
@@ -631,8 +678,39 @@ def get_user_env_aws_account(region: str) -> str:
     return boto3.client("sts", region_name=region).get_caller_identity()["Account"]
 
 
+def generate_inline_policy_parameter(policy: AnyscaleIAMPolicy) -> str:
+    """Generate the inline policy paramter for the cross account role"""
+    return f"""  {policy.parameter_key}:
+    Description: {policy.parameter_description}
+    Type: String"""
+
+
+def generate_inline_policy_resource(policy: AnyscaleIAMPolicy) -> str:
+    """Generate the inline policy resource for the cross account role"""
+    return f"""  {policy.resource_logical_id}:
+    Type: AWS::IAM::Policy
+    Properties:
+      PolicyDocument: !Ref {policy.parameter_key}
+      PolicyName: {policy.policy_name}
+      Roles:
+        - !Ref customerRole"""
+
+
+def get_anyscale_cross_account_iam_policies() -> List[Dict[str, str]]:
+    return [
+        {
+            "ParameterKey": policy.parameter_key,
+            "ParameterValue": policy.policy_document,
+        }
+        for policy in ANYSCALE_IAM_POLICIES
+    ]
+
+
 def prepare_cloudformation_template(
-    region: str, cfn_stack_name: str, cloud_id: str
+    region: str,
+    cfn_stack_name: str,
+    cloud_id: str,
+    enable_head_node_fault_tolerance: bool,
 ) -> str:
     with open(f"{anyscale.conf.ROOT_DIR_PATH}/anyscale-cloud-setup.yaml") as f:
         body = f.read()
@@ -701,6 +779,78 @@ def prepare_cloudformation_template(
     body = body.replace("$ALLOWED_ORIGIN", shared_anyscale_conf.ANYSCALE_HOST)
     body = body.replace("$EFSMountTargets", "\n".join(efs_mount_targets))
 
+    iam_policy_parameters = [
+        generate_inline_policy_parameter(policy) for policy in ANYSCALE_IAM_POLICIES
+    ]
+    body = body.replace(
+        "$ANYSCALE_CROSS_ACCOUNT_IAM_POLICY_PARAMETERS",
+        "\n\n".join(iam_policy_parameters),
+    )
+
+    iam_policy_resources = [
+        generate_inline_policy_resource(policy) for policy in ANYSCALE_IAM_POLICIES
+    ]
+    body = body.replace(
+        "$ANYSCALE_CROSS_ACCOUNT_IAM_POLICY_RESOURCES",
+        "\n\n".join(iam_policy_resources),
+    )
+
+    if enable_head_node_fault_tolerance:
+        body = body.replace(
+            "$MEMORY_DB_OUTPUT",
+            """  MemoryDB:
+    Description: MemoryDB cluster
+    Value:
+      Fn::ToJsonString:
+        arn: !GetAtt MemoryDB.ARN
+        ClusterEndpointAddress: !GetAtt MemoryDB.ClusterEndpoint.Address""",
+        )
+        # chr(10) is a newline (\n) since f-string expression part cannot include a backslash
+        body = body.replace(
+            "$MEMORY_DB_RESOURCE",
+            f"""  MemoryDBSubnetGroup:
+    Type: AWS::MemoryDB::SubnetGroup
+    Properties:
+      Description: Anyscale managed MemoryDB subnet group
+      SubnetGroupName: !Ref AWS::StackName
+      SubnetIds:
+{chr(10).join([f"        - !Ref Subnet{i}" for i in range(len(azs))])}
+      Tags:
+        - Key: anyscale-cloud-id
+          Value: !Ref CloudID
+
+  MemoryDBParameterGroup:
+    Type: AWS::MemoryDB::ParameterGroup
+    Properties:
+      Description: Parameter group for anyscale managed MemoryDB
+      Family: memorydb_redis7
+      ParameterGroupName:  !Ref AWS::StackName
+      Tags:
+        - Key: anyscale-cloud-id
+          Value: !Ref CloudID
+
+  MemoryDB:
+    Type: AWS::MemoryDB::Cluster
+    Properties:
+      ACLName: open-access
+      Description: Anyscale managed MemoryDB
+      ClusterName: !Ref AWS::StackName
+      NodeType: db.t4g.small
+      Port: !Ref MemoryDBRedisPort
+      SubnetGroupName: !Ref MemoryDBSubnetGroup
+      SecurityGroupIds:
+        - !Ref AnyscaleSecurityGroup
+      EngineVersion: "7.0"
+      ParameterGroupName: !Ref MemoryDBParameterGroup
+      TLSEnabled: false
+      Tags:
+        - Key: anyscale-cloud-id
+          Value: !Ref CloudID""",
+        )
+    else:
+        body = body.replace("$MEMORY_DB_OUTPUT", "")
+        body = body.replace("$MEMORY_DB_RESOURCE", "")
+
     return body
 
 
@@ -759,7 +909,9 @@ def validate_job_config_dict(
         config_dict["compute_config"], str
     ):
         compute_config = get_cluster_compute_from_name(
-            config_dict["compute_config"], api_client
+            config_dict["compute_config"],
+            api_client,
+            config_dict.get("compute_config_version"),
         )
     elif "compute_config_id" in config_dict:
         cluster_compute_id = config_dict["compute_config_id"]
