@@ -1,6 +1,5 @@
 import contextlib
 from enum import Enum
-import json
 import sys
 from typing import Optional, Tuple
 import uuid
@@ -127,6 +126,8 @@ class CloudSetupError(str, Enum):
     FIREWALL_NOT_ASSOCIATED_WITH_VPC = "FIREWALL_NOT_ASSOCIATED_WITH_VPC"
     FILESTORE_NAME_MALFORMED = "FILESTORE_NAME_MALFORMED"
     FILESTORE_NOT_CONNECTED_TO_VPC = "FILESTORE_NOT_CONNECTED_TO_VPC"
+    MEMORYSTORE_NOT_CONNECTED_TO_VPC = "MEMORYSTORE_NOT_CONNECTED_TO_VPC"
+    MEMORYSTORE_READ_REPLICAS_DISABLED = "MEMORYSTORE_READ_REPLICAS_DISABLED"
 
 
 class CloudEventProducer:
@@ -137,16 +138,19 @@ class CloudEventProducer:
     def __init__(self, api_client: DefaultApi):
         self.api_client = api_client
         self.cloud_id: Optional[str] = None
+        self.cloud_provider: Optional[CloudProviders] = None
 
     def init_trace_context(
         self,
         command_name: CloudAnalyticsEventCommandName,
+        cloud_provider: CloudProviders,
         cloud_id: Optional[str] = None,
     ):
         self.trace_id = str(uuid.uuid4().hex)
         self.command_name = command_name
         self.raw_command_input = str(" ".join(sys.argv[1:]))
         self.cloud_id = cloud_id
+        self.cloud_provider = cloud_provider
 
     def set_cloud_id(self, cloud_id: str):
         self.cloud_id = cloud_id
@@ -162,41 +166,16 @@ class CloudEventProducer:
             # shouldn't block cloud setup even if cloud event generation fails
             error = None
             if not succeeded:
-                # populate error
+                cloud_provider_error = None
                 if logger:
-                    cloud_provider_errors = logger.get_cloud_provider_errors()
+                    cloud_provider_error_list = logger.get_cloud_provider_errors()
                     logger.clear_cloud_provider_errors()
-                if len(cloud_provider_errors) > 0:
-                    if internal_error:
-                        # NOTE: Both internal errors and cloud provider errors are populated.
-                        # This shouldn't happen, but if it does, we want to log both.
-                        # We will merge the errors since the backend will only allow one field to be populated.
-                        error_msg = json.dumps(
-                            {
-                                "cloud_provider_errors": [
-                                    {
-                                        "cloud_resource": str(
-                                            cloud_provider_error.cloud_resource
-                                        ),
-                                        "error_code": str(
-                                            cloud_provider_error.error_code
-                                        ),
-                                    }
-                                    for cloud_provider_error in cloud_provider_errors
-                                ],
-                                "internal_error": internal_error,
-                            }
-                        )
-                        error = CloudAnalyticsEventError(internal_error=error_msg)
-                    else:
-                        error = CloudAnalyticsEventError(
-                            cloud_provider_error=cloud_provider_errors
-                        )
-                else:
-                    # not cloud provider error
-                    error = CloudAnalyticsEventError(
-                        internal_error=internal_error if internal_error else "",
-                    )
+                    if len(cloud_provider_error_list) > 0:
+                        cloud_provider_error = cloud_provider_error_list
+                error = CloudAnalyticsEventError(
+                    internal_error=internal_error,
+                    cloud_provider_error=cloud_provider_error,
+                )
 
             self.api_client.produce_analytics_event_api_v2_analytics_post(
                 CreateAnalyticsEvent(
@@ -206,6 +185,7 @@ class CloudEventProducer:
                         succeeded=succeeded,
                         command_name=self.command_name,
                         raw_command_input=self.raw_command_input,
+                        cloud_provider=self.cloud_provider,
                         event_name=event_name,
                         error=error,
                     )
