@@ -75,7 +75,10 @@ def enable_project_apis(
         )
     except HttpError as e:
         logger.log_resource_exception(CloudAnalyticsEventCloudResource.GCP_PROJECT, e)
-        raise ClickException(f"Failed to enable APIs for project {project_id}: {e}")
+        raise ClickException(
+            f"Failed to enable APIs for project {project_id}: {e}. "
+            f"Please make sure the service usage API is enabled: https://console.cloud.google.com/marketplace/product/google/serviceusage.googleapis.com?project={project_id}"
+        )
 
 
 def get_workload_identity_pool(
@@ -142,6 +145,7 @@ def get_deployment_resources(
     # TODO (congding): use a pydantic model instead of dict
     cloud_resources = {}
     for resource in config["resources"]:
+        resource_props = resource["properties"]
         if resource["name"] == anyscale_access_service_account_name:
             # we should just keep the cluster service account and skip this
             continue
@@ -150,10 +154,16 @@ def get_deployment_resources(
             continue
         if resource["type"] == "gcp-types/file-v1beta1:projects.locations.instances":
             # get filestore location and instance
-            cloud_resources["filestore_instance"] = resource["properties"]["instanceId"]
-            cloud_resources["filestore_location"] = resource["properties"][
-                "parent"
-            ].split("/")[-1]
+            cloud_resources["filestore_instance"] = resource_props["instanceId"]
+            cloud_resources["filestore_location"] = resource_props["parent"].split("/")[
+                -1
+            ]
+        if resource["type"] == "gcp-types/redis-v1:projects.locations.instances":
+            # get redis instance
+            cloud_resources[
+                "memorystore_name"
+            ] = f'{resource_props["parent"]}/instances/{resource_props["instanceId"]}'
+
         cloud_resources[resource["type"]] = resource["name"]
 
     return cloud_resources
@@ -269,6 +279,7 @@ def generate_deployment_manager_config(  # noqa: PLR0913
     workload_identity_pool_name: str,
     anyscale_aws_account: str,
     organization_id: str,
+    enable_head_node_fault_tolerance: bool,
     subnet_cidr: str = "10.0.0.0/20",
     zone: str = "b",  # some regions like us-east1 or europe-west1 don't have zone "a"
     fileshare_capacity_gb: int = 1024,
@@ -278,7 +289,29 @@ def generate_deployment_manager_config(  # noqa: PLR0913
     with open(
         os.path.join(anyscale.conf.ROOT_DIR_PATH, "anyscale-cloud-setup-gcp.yaml")
     ) as f:
-        deployment_manager_config = Template(f.read())
+        body = f.read()
+
+    if enable_head_node_fault_tolerance:
+        body += """
+- name: redis-${CLOUD_ID}
+  type: gcp-types/redis-v1:projects.locations.instances
+  properties:
+    parent: projects/${PROJECT_ID}/locations/${REGION}
+    instanceId: redis-${CLOUD_ID}
+    authorizedNetwork: projects/${PROJECT_ID}/global/networks/$$(ref.vpc-${CLOUD_ID}.name)
+    memorySizeGb: 5
+    replicaCount: 1
+    readReplicasMode: READ_REPLICAS_ENABLED
+    tier: STANDARD_HA
+    redisVersion: REDIS_7_0
+    displayName: redis-${CLOUD_ID}
+    redisConfigs:
+        maxmemory-policy: allkeys-lru
+  metadata:
+   dependsOn:
+     - subnet-${CLOUD_ID}
+"""
+    deployment_manager_config = Template(body)
 
     params = {
         "ANYSCALE_HOST": shared_anyscale_conf.ANYSCALE_HOST,
