@@ -5,7 +5,7 @@ import subprocess
 from typing import Any, Dict, List, Optional, Set, Tuple, Union
 
 from click import ClickException
-from google.api_core.exceptions import NotFound
+from google.api_core.exceptions import Forbidden, NotFound, PermissionDenied
 import google.auth
 from google.auth.credentials import Credentials, CredentialsWithQuotaProject
 from google.auth.exceptions import DefaultCredentialsError, RefreshError
@@ -23,6 +23,9 @@ from anyscale.client.openapi_client.models.cloud_analytics_event_cloud_resource
 from anyscale.client.openapi_client.models.gcp_file_store_config import (
     GCPFileStoreConfig,
 )
+from anyscale.client.openapi_client.models.gcp_memorystore_instance_config import (
+    GCPMemorystoreInstanceConfig,
+)
 from anyscale.util import confirm
 from anyscale.utils.cloud_utils import CloudSetupError
 
@@ -35,6 +38,7 @@ GCP_REQUIRED_APIS = [
     "deploymentmanager.googleapis.com",  # Deployment Manager
     "cloudresourcemanager.googleapis.com",  # Resource Manager
     "certificatemanager.googleapis.com",  # Certificate Manager for Services
+    "redis.googleapis.com",  # Memorystore for Redis
 ]
 
 GCP_RESOURCE_DICT: Dict[str, CloudAnalyticsEventCloudResource] = {
@@ -46,6 +50,7 @@ GCP_RESOURCE_DICT: Dict[str, CloudAnalyticsEventCloudResource] = {
     "Firewall Policy": CloudAnalyticsEventCloudResource.GCP_FIREWALL_POLICY,
     "Filestore": CloudAnalyticsEventCloudResource.GCP_FILESTORE,
     "Google Cloud Storage Bucket": CloudAnalyticsEventCloudResource.GCP_STORAGE_BUCKET,
+    "Memorystore": CloudAnalyticsEventCloudResource.GCP_MEMORYSTORE,
 }
 
 
@@ -279,3 +284,25 @@ def get_filestore_location_and_instance_id(
         raise ClickException(
             f"Could not parse Filestore instance name {instance_name}."
         )
+
+
+def get_gcp_memorystore_config(
+    factory: GoogleCloudClientFactory, instance_name: Optional[str]
+) -> Optional[GCPMemorystoreInstanceConfig]:
+    """Get the Memorystore instance config from GCP
+    Returns None if instance_name is None
+    """
+    if not instance_name:
+        return None
+
+    client = factory.redis_v1.CloudRedisClient()
+
+    try:
+        instance = client.get_instance(name=instance_name)
+    except (NotFound, PermissionDenied, Forbidden) as e:
+        raise ClickException(
+            f"Error occurred when trying to access the memorystore instance {instance_name}: {e}.\nPlease validate that you're using the correct GCP project and that the resource values are correct."
+        )
+    return GCPMemorystoreInstanceConfig(
+        name=instance_name, endpoint=instance.host + ":" + str(instance.port),
+    )
