@@ -244,9 +244,15 @@ def test_get_cluster_compute(mock_auth_api_client):
         )
 
 
+@pytest.mark.parametrize("compute_config_version", [None, 1])
 @pytest.mark.parametrize("needs_start", [True, False])
 @pytest.mark.parametrize("cluster_exists", [True, False])
-def test_start(mock_auth_api_client, needs_start: bool, cluster_exists: bool) -> None:
+def test_start(
+    mock_auth_api_client,
+    needs_start: bool,
+    cluster_exists: bool,
+    compute_config_version: Optional[int],
+) -> None:
     cluster_controller = ClusterController()
     mock_build = Mock(id="build_id")
     mock_cluster_compute = Mock(id="cluster_compute_id")
@@ -295,6 +301,7 @@ def test_start(mock_auth_api_client, needs_start: bool, cluster_exists: bool) ->
             project_id=None,
             project_name=None,
             user_service_access=mock_user_service_access,
+            cluster_compute_version=compute_config_version,
         )
 
     cluster_controller._get_project_id_and_cluster_name.assert_called_once_with(
@@ -309,7 +316,7 @@ def test_start(mock_auth_api_client, needs_start: bool, cluster_exists: bool) ->
         cloud_name=None,
     )
     mock_get_cluster_compute_from_name.assert_called_once_with(
-        "cluster_compute_name", cluster_controller.api_client
+        "cluster_compute_name", cluster_controller.api_client, compute_config_version
     )
     cluster_controller.anyscale_api_client.search_clusters.assert_called_once_with(
         {
@@ -322,7 +329,12 @@ def test_start(mock_auth_api_client, needs_start: bool, cluster_exists: bool) ->
         "cluster_env_name"
     )
     cluster_controller._get_cluster_compute.assert_called_once_with(
-        "cluster_compute_name", None, None, mock_existing_cluster, "mock_project_id"
+        "cluster_compute_name",
+        None,
+        None,
+        mock_existing_cluster,
+        "mock_project_id",
+        compute_config_version,
     )
     cluster_controller._create_or_update_cluster_data.assert_called_once_with(
         mock_existing_cluster,
