@@ -148,6 +148,7 @@ def test_config_validation(add_extra_field, mock_auth_api_client):
         "project_id": project_id,
         "build_id": build_id,
         "compute_config_id": compute_config_id,
+        "compute_config_version": 2,
     }
     if add_extra_field:
         extra_field_value = "*****"
@@ -283,6 +284,7 @@ def test_submit_job(
         cluster_compute_id=compute_template_test_data.id,
         cluster_compute=config_dict.get("compute_config"),
         cloud=config_dict.get("cloud"),
+        cluster_compute_version=config_dict.get("compute_config_version"),
     )
 
     if config_project_id:
@@ -312,7 +314,9 @@ def test_submit_job(
     if "compute_config" in config_dict:
         compute_config = config_dict["compute_config"]
         if isinstance(compute_config, str):
-            mock_get_cluster_compute_from_name.assert_called_with(compute_config)
+            mock_get_cluster_compute_from_name.assert_called_with(
+                compute_config, version=None
+            )
         elif isinstance(compute_config, dict):
             mock_register_compute_template.assert_called_once_with(compute_config)
     elif "cloud" in config_dict:
@@ -815,7 +819,7 @@ class TestComputeConfig:
         assert jc.compute_config_id == "mock_existing_compute_config_id"
         mock_get_default_cluster_compute.assert_not_called()
         mock_get_cluster_compute_from_name.assert_called_once_with(
-            "mock_existing_compute_config_name"
+            "mock_existing_compute_config_name", version=None
         )
         mock_register_compute_template.assert_not_called()
         mock_get_parent_cloud_id_and_name_of_project.assert_not_called()
