@@ -2,7 +2,7 @@ import os
 from pathlib import Path
 import tempfile
 from types import SimpleNamespace
-from typing import List
+from typing import Any, List
 from unittest.mock import Mock, patch
 
 import click
@@ -12,9 +12,7 @@ import yaml
 from anyscale.controllers.job_controller import LogsLogger
 from anyscale.controllers.schedule_controller import (
     assert_single_parameter,
-    CreateScheduleConfig,
     load_yaml_file_with_overrides,
-    ScheduleConfig,
     ScheduleController,
 )
 
@@ -112,24 +110,6 @@ def test_load_yaml_file(schedule_yaml_file, mock_auth_api_client):
 PIP_LIST = ["requests==1.0.0", "pip-install-test"]
 
 
-def test_rewrite_re(mock_auth_api_client, patch_jobs_anyscale_api_client):
-    with patch.multiple(
-        "anyscale.controllers.schedule_controller",
-        override_runtime_env_for_local_working_dir=Mock(
-            return_value={"rewritten": True}
-        ),
-    ):
-        config = CreateScheduleConfig(
-            entrypoint="ls",
-            build_id="123",
-            compute_config_id="test",
-            runtime_env={"pip": PIP_LIST},
-            schedule=ScheduleConfig(cron_expression="* * * * *"),
-            project_id="test_project_id",
-        )
-        assert config.runtime_env == {"rewritten": True}
-
-
 def test_apply_schedule(
     schedule_yaml_file, mock_auth_api_client, patch_jobs_anyscale_api_client
 ):
@@ -137,11 +117,20 @@ def test_apply_schedule(
     schedule_controller.api_client.get_compute_template_api_v2_compute_templates_template_id_get = Mock(
         return_value=Mock(result=Mock(archived_at=False))
     )
+
+    def infer_project_id_mock(*args: Any, **kwargs: Any):
+        if kwargs.get("project_id"):
+            return kwargs.get("project_id")
+        else:
+            return "mock_project_id"
+
+    mock_infer_project_id = Mock(side_effect=infer_project_id_mock)
+    mock_override_runtime_env_config = Mock(return_value=None)
+
     with patch.multiple(
         "anyscale.controllers.schedule_controller",
-        override_runtime_env_for_local_working_dir=Mock(
-            return_value={"rewritten": True}
-        ),
+        override_runtime_env_config=mock_override_runtime_env_config,
+        infer_project_id=mock_infer_project_id,
     ):
         schedule_controller.apply(schedule_yaml_file, "override_name")
         args = schedule_controller.api_client.create_or_update_job_api_v2_experimental_cron_jobs_put.call_args[
@@ -157,7 +146,7 @@ def test_apply_schedule(
                 "entrypoint": "ls",
                 "max_retries": 5,
                 "ray_serve_config": None,
-                "runtime_env": {"rewritten": True},
+                "runtime_env": None,
                 "runtime_env_config": None,
             },
             "description": None,
@@ -255,9 +244,7 @@ def test_resolve_file(
     )
     with patch.multiple(
         "anyscale.controllers.schedule_controller",
-        override_runtime_env_for_local_working_dir=Mock(
-            return_value={"rewritten": True}
-        ),
+        override_runtime_env_config=Mock(return_value={"rewritten": True}),
     ):
         schedule_controller.schedule_api = Mock()
         schedule_controller.schedule_api.list.return_value = [
@@ -281,3 +268,47 @@ def test_trigger(mock_auth_api_client):
     schedule_controller.schedule_api = Mock()
     schedule_controller.trigger(id="id")
     schedule_controller.schedule_api.trigger.assert_called_once()
+
+
+def test_override_runtime_env_config_and_infer_project_id(
+    mock_auth_api_client, schedule_yaml_file
+):
+    """Test helper functions in _resolve_config
+    Test that runtime_env is overwrriten from override_runtime_env_config
+    Test the project_id is autopopulated from infer_project_id
+    """
+    new_config = {
+        "name": "schedule",
+        "entrypoint": "ls",
+        "build_id": "build_id",
+        "compute_config_id": "cc_id",
+        "schedule": {"cron_expression": "ce"},
+        "runtime_env": {"pip": ["requests"]},
+    }
+    schedule_controller = ScheduleController()
+
+    mock_project_id = "default"
+
+    mock_infer_project_id = Mock(return_value=mock_project_id)
+
+    new_runtime_env = {"working_dir": "s3://bucket"}
+    mock_override_runtime_env_config = Mock(return_value=new_runtime_env)
+
+    mock_load_yaml_file_with_overrides = Mock(
+        return_value=load_yaml_file_with_overrides(schedule_yaml_file, **new_config)
+    )
+
+    with patch.multiple(
+        "anyscale.controllers.schedule_controller",
+        validate_job_config_dict=Mock(),
+        infer_project_id=mock_infer_project_id,
+        override_runtime_env_config=mock_override_runtime_env_config,
+        load_yaml_file_with_overrides=mock_load_yaml_file_with_overrides,
+    ), patch.multiple(
+        "anyscale.models.job_model", validate_successful_build=Mock(),
+    ):
+        schedule_config = schedule_controller._resolve_config(schedule_yaml_file)
+
+        assert schedule_config.project_id == mock_project_id
+
+        assert schedule_config.runtime_env == new_runtime_env
