@@ -17,7 +17,8 @@ from anyscale.client.openapi_client.models.decoratedlistserviceapimodel_list_res
 from anyscale.client.openapi_client.models.list_response_metadata import (
     ListResponseMetadata,
 )
-from anyscale.controllers.service_controller import ServiceConfig, ServiceController
+from anyscale.controllers.service_controller import ServiceController
+from anyscale.models.service_model import ServiceConfig
 from anyscale.utils.workload_types import Workload
 
 
@@ -337,6 +338,7 @@ def test_service_rollout(
     compute_config_id = "test_compute_config_id"
     project_id = "test_project_id"
     rollout_strategy = "ROLLOUT"
+    service_config = {"access": {"use_bearer_token": True}}
 
     config_dict = {
         "name": name,
@@ -348,6 +350,7 @@ def test_service_rollout(
         "version": "test_abc",
         "canary_percent": 100,
         "rollout_strategy": rollout_strategy,
+        "config": service_config,
     }
     mock_validate_successful_build = Mock()
     mock_override_runtime_env_config = Mock(
@@ -384,6 +387,7 @@ def test_service_rollout(
         rollout_strategy=override_rollout_strategy
         if override_rollout_strategy
         else "ROLLOUT",
+        config=service_config,
     )
 
     service_controller.api_client.apply_service_v2_api_v2_services_v2_apply_put.assert_called_once_with(
