@@ -8,6 +8,8 @@ from typing import Any, Callable, Dict, List, Optional
 from unittest.mock import Mock, patch
 
 import boto3
+from botocore.exceptions import ClientError
+from click import ClickException
 from moto import mock_cloudformation, mock_ec2, mock_efs, mock_iam, mock_s3
 import pytest
 
@@ -20,6 +22,9 @@ from anyscale.aws_iam_policies import (
     get_anyscale_aws_iam_assume_role_policy,
 )
 from anyscale.cli_logger import CloudSetupLogger
+from anyscale.client.openapi_client.models.aws_memory_db_cluster_config import (
+    AWSMemoryDBClusterConfig,
+)
 from anyscale.client.openapi_client.models.create_cloud_resource import (
     CreateCloudResource,
 )
@@ -30,6 +35,7 @@ from anyscale.cloud_resource import (
     verify_aws_cloudformation_stack,
     verify_aws_efs,
     verify_aws_iam_roles,
+    verify_aws_memorydb_cluster,
     verify_aws_s3,
     verify_aws_security_groups,
     verify_aws_subnets,
@@ -96,7 +102,9 @@ def _delete_role(role_arn: str) -> None:
     iam.delete_role(RoleName=role_name)
 
 
-def generate_cloud_resource_mock_aws() -> CreateCloudResource:
+def generate_cloud_resource_mock_aws(
+    enable_head_node_fault_tolerance: bool = False,
+) -> CreateCloudResource:
     return CreateCloudResource(
         aws_vpc_id="fake_aws_vpc_id",
         aws_subnet_ids_with_availability_zones=[
@@ -115,6 +123,11 @@ def generate_cloud_resource_mock_aws() -> CreateCloudResource:
         aws_s3_id="fake_aws_s3_id",
         aws_efs_id="fake_aws_efs_id",
         aws_cloudformation_stack_id="fake_aws_cloudformation_stack_id",
+        memorydb_cluster_config=AWSMemoryDBClusterConfig(
+            id="fake_memorydb_id", endpoint="fake_memorydb_endpoint:6379"
+        )
+        if enable_head_node_fault_tolerance
+        else None,
     )
 
 
@@ -1264,3 +1277,285 @@ def test_verify_aws_cloudformation_stack(
         logger=CloudSetupLogger(),
     )
     assert result == expected_result
+
+
+@pytest.mark.parametrize(
+    (
+        "enable_head_node_fault_tolerance",
+        "redis_cluster_exist",
+        "matching_security_group",
+        "matching_vpc",
+        "matching_subnets",
+        "has_lru_set",
+        "tls_disabled",
+        "is_ha",
+        "should_raise_exception",
+        "expected_result",
+        "expected_log_message",
+    ),
+    [
+        pytest.param(
+            False,
+            False,
+            False,
+            False,
+            False,
+            False,
+            False,
+            False,
+            False,
+            False,
+            "Missing MemoryDB cluster id",
+        ),
+        pytest.param(
+            True,
+            False,
+            False,
+            False,
+            False,
+            False,
+            False,
+            False,
+            False,
+            False,
+            "Could not find MemoryDB cluster",
+        ),
+        pytest.param(
+            True,
+            True,
+            False,
+            False,
+            False,
+            False,
+            False,
+            False,
+            False,
+            False,
+            "that is not the same as the cloud's security group",
+        ),
+        pytest.param(
+            True,
+            True,
+            True,
+            False,
+            False,
+            False,
+            False,
+            False,
+            False,
+            False,
+            "is not in the same VPC as the cloud",
+        ),
+        pytest.param(
+            True,
+            True,
+            True,
+            True,
+            False,
+            False,
+            False,
+            False,
+            False,
+            False,
+            "that is not one of the subnets in the cloud",
+        ),
+        pytest.param(
+            True,
+            True,
+            True,
+            True,
+            True,
+            False,
+            False,
+            False,
+            False,
+            False,
+            "should have parameter group with maxmemory-policy set to allkeys-lru",
+        ),
+        pytest.param(
+            True,
+            True,
+            True,
+            True,
+            True,
+            True,
+            False,
+            False,
+            False,
+            False,
+            "has TLS enabled which is not supported by ray",
+        ),
+        pytest.param(
+            True,
+            True,
+            True,
+            True,
+            True,
+            True,
+            True,
+            False,
+            False,
+            False,
+            "This is not enough for high availability",
+        ),
+        pytest.param(
+            True,
+            True,
+            True,
+            True,
+            True,
+            True,
+            True,
+            True,
+            True,
+            False,
+            "Failed to verify MemoryDB cluster",
+        ),
+        pytest.param(True, True, True, True, True, True, True, True, False, True, None),
+    ],
+)
+def test_verify_aws_memorydb_cluster(  # noqa: PLR0913
+    capsys,
+    enable_head_node_fault_tolerance,
+    redis_cluster_exist,
+    matching_security_group,
+    matching_vpc,
+    matching_subnets,
+    has_lru_set,
+    should_raise_exception,
+    tls_disabled,
+    is_ha,
+    expected_result,
+    expected_log_message,
+):
+    cloud_resource_mock = generate_cloud_resource_mock_aws(
+        enable_head_node_fault_tolerance=enable_head_node_fault_tolerance
+    )
+
+    if not redis_cluster_exist:
+        mock_cluster_resp = {"Clusters": []}
+    else:
+        mock_cluster_resp = {
+            "Clusters": [
+                {
+                    "SecurityGroups": [
+                        {
+                            "SecurityGroupId": cloud_resource_mock.aws_security_groups[
+                                0
+                            ]
+                            if matching_security_group
+                            else "not-matching-security-group"
+                        }
+                    ],
+                    "SubnetGroupName": "subnet-group-1234567890",
+                    "ParameterGroupName": "parameter-group-1234567890",
+                    "TLSEnabled": True,
+                    "Shards": [
+                        {
+                            "Name": "shard-0001",
+                            "Nodes": [
+                                {
+                                    "Name": "node-0001",
+                                    "Status": "available",
+                                    "Endpoint": "node-0001.fake.com",
+                                }
+                            ],
+                        }
+                    ],
+                },
+            ]
+        }
+
+    mock_subnet_group_resp = {
+        "SubnetGroups": [
+            {
+                "SubnetGroupName": "subnet-group-1234567890",
+                "VpcId": cloud_resource_mock.aws_vpc_id
+                if matching_vpc
+                else "not-matching-vpc",
+            }
+        ]
+    }
+
+    if matching_subnets:
+        mock_subnet_group_resp["SubnetGroups"][0]["Subnets"] = [
+            {
+                "Identifier": cloud_resource_mock.aws_subnet_ids_with_availability_zones[
+                    0
+                ].subnet_id,
+            },
+            {
+                "Identifier": cloud_resource_mock.aws_subnet_ids_with_availability_zones[
+                    1
+                ].subnet_id,
+            },
+        ]
+    else:
+        mock_subnet_group_resp["SubnetGroups"][0]["Subnets"] = [
+            {"Identifier": "not-matching-subnet-1",},
+            {"Identifier": "not-matching-subnet-2",},
+        ]
+
+    if has_lru_set:
+        mock_parameter_group_resp = {
+            "Parameters": [{"Name": "maxmemory-policy", "Value": "allkeys-lru",}]
+        }
+    else:
+        mock_parameter_group_resp = {
+            "Parameters": [{"Name": "maxmemory-policy", "Value": "not-allkeys-lru",}]
+        }
+
+    if tls_disabled:
+        mock_cluster_resp["Clusters"][0]["TLSEnabled"] = False
+
+    if is_ha:
+        mock_cluster_resp["Clusters"][0]["Shards"][0]["Nodes"].append(
+            {
+                "Name": "node-0002",
+                "Status": "available",
+                "Endpoint": "node-0002.fake.com",
+            }
+        )
+
+    mock_memorydb_client = Mock(
+        describe_clusters=Mock(return_value=mock_cluster_resp),
+        describe_subnet_groups=Mock(return_value=mock_subnet_group_resp),
+        describe_parameters=Mock(return_value=mock_parameter_group_resp),
+    )
+    if should_raise_exception:
+        mock_memorydb_client.describe_clusters.side_effect = ClientError(
+            {
+                "Error": {
+                    "Code": "AccessDeniedException",
+                    "Message": "Permission denied",
+                }
+            },
+            "describe_clusters",
+        )
+
+    mock_boto3 = Mock(client=Mock(return_value=mock_memorydb_client))
+
+    if should_raise_exception:
+        with pytest.raises(ClickException, match=expected_log_message):
+            verify_aws_memorydb_cluster(
+                cloud_resource=cloud_resource_mock,
+                boto3_session=mock_boto3,
+                logger=CloudSetupLogger(),
+                strict=True,
+            )
+        return
+
+    result = verify_aws_memorydb_cluster(
+        cloud_resource=cloud_resource_mock,
+        boto3_session=mock_boto3,
+        logger=CloudSetupLogger(),
+        strict=True,
+    )
+
+    stdout, stderr = capsys.readouterr()
+    sys.stdout.write(stdout)
+    sys.stderr.write(stderr)
+
+    assert result == expected_result
+    if expected_log_message:
+        assert re.search(expected_log_message, stderr)
