@@ -275,7 +275,7 @@ def _make_test_builder(
     builder._start_session.side_effect = create_session  # type: ignore
     builder._register_compute_template = Mock(return_value="mock_compute_template_id")  # type: ignore
 
-    builder._get_last_used_cloud = Mock(return_value="anyscale_default_cloud")  # type: ignore
+    builder._get_last_used_cloud = Mock(return_value="anyscale_v2_default_cloud")  # type: ignore
 
     if create_build:
         _make_app_template()
@@ -610,6 +610,7 @@ def test_new_proj_connect_params(
         cloud_name=None,
         cluster_compute_dict=None,
         cluster_compute_name=None,
+        cluster_compute_version=None,
     )
     mock_get_prepare_cluster_block.assert_called_once_with(
         allow_public_internet_traffic=builder._allow_public_internet_traffic,
@@ -628,6 +629,7 @@ def test_new_proj_connect_params(
         project_id=mock_project_block.project_id,
         ray=builder._ray,
         log_output=True,
+        cluster_compute_version=builder._cluster_compute_version,
     )
     mock_get_start_interactive_session_block.assert_called_once_with(
         cluster=mock_cluster,
@@ -715,6 +717,7 @@ def test_connect_with_cloud(
         project_id=mock_get_project_block.return_value.project_id,
         ray=builder._ray,
         log_output=True,
+        cluster_compute_version=builder._cluster_compute_version,
     )
 
 
@@ -757,6 +760,7 @@ def test_connect_with_cluster_env_dict(
         project_id=mock_get_project_block.return_value.project_id,
         ray=builder._ray,
         log_output=True,
+        cluster_compute_version=builder._cluster_compute_version,
     )
 
 
@@ -1112,6 +1116,7 @@ def test_set_runtime_env_in_job_config(
         project_id=mock_get_project_block.return_value.project_id,
         ray=connect_instance._ray,
         log_output=True,
+        cluster_compute_version=connect_instance._cluster_compute_version,
     )
     mock_get_start_interactive_session_block.assert_called_once_with(
         cluster=mock_cluster,
