@@ -749,7 +749,8 @@ def test_remove_firewall_policy_associations(
         )
 
 
-def test_generate_deployment_manager_config():
+@pytest.mark.parametrize("enable_head_node_fault_tolerance", [True, False])
+def test_generate_deployment_manager_config(enable_head_node_fault_tolerance):
     mock_region = "mock-region"
     mock_project_id = "mock-project"
     mock_cloud_id = "cld_mock"
@@ -768,10 +769,16 @@ def test_generate_deployment_manager_config():
         workload_identity_pool_name,
         anyscale_aws_account,
         organization_id,
+        enable_head_node_fault_tolerance,
     )
 
-    # verify it's a valid yaml file
+    # # verify it's a valid yaml file
     yaml.safe_load(generated_config)
 
     # verify all ${val} are substituted
     assert re.search(r"\$\{.*\}", generated_config) is None
+
+    if enable_head_node_fault_tolerance:
+        assert "gcp-types/redis-v1:projects.locations.instances" in generated_config
+    else:
+        assert "gcp-types/redis-v1:projects.locations.instances" not in generated_config
