@@ -20,11 +20,15 @@ from anyscale.client.openapi_client.models import CreateCloudResourceGCP
 from anyscale.client.openapi_client.models.gcp_file_store_config import (
     GCPFileStoreConfig,
 )
+from anyscale.client.openapi_client.models.gcp_memorystore_instance_config import (
+    GCPMemorystoreInstanceConfig,
+)
 from anyscale.gcp_verification import (
     _check_bucket_region,
     _firewall_rules_from_proto_resp,
     _gcp_subnet_has_enough_capacity,
     _verify_service_account_on_bucket,
+    _vpc_has_proxy_only_subnet,
     GCPLogger,
     verify_cloud_storage,
     verify_filestore,
@@ -33,9 +37,9 @@ from anyscale.gcp_verification import (
     verify_gcp_dataplane_service_account,
     verify_gcp_networking,
     verify_gcp_project,
+    verify_memorystore,
 )
 from anyscale.utils.network_verification import Direction, FirewallRule, Protocol
-from frontend.cli.anyscale.gcp_verification import _vpc_has_proxy_only_subnet
 
 
 _MOCK_GCP_PROJECT_ID = "anyscale-bridge-deadbeef"
@@ -56,6 +60,10 @@ def generate_gcp_cloud_mock_resources(**overrides: Any) -> CreateCloudResourceGC
             mount_target_ip="mock_ip",
         ),
         "gcp_cloud_storage_bucket_id": "cloud-bucket",
+        "memorystore_instance_config": GCPMemorystoreInstanceConfig(
+            name="projects/anyscale-bridge-deadbeef/locations/us-west1/instances/memorystore-one",
+            endpoint="mock_ip",
+        ),
     }
     return CreateCloudResourceGCP(**{**default_resources, **overrides})
 
@@ -635,6 +643,48 @@ def test_verify_cloud_storage(setup_mock_server, responses, expected_result: boo
     )
 
 
+@pytest.mark.parametrize(
+    ("responses", "expected_result"),
+    [
+        pytest.param([("404 Not Found", None)], False, id="NoInstance"),
+        pytest.param(
+            [("200 OK", "regional_memorystore.json"),], False, id="VPCMismatch",
+        ),
+        pytest.param(
+            [("200 OK", "regional_memorystore.json"),], False, id="tierMismatch",
+        ),
+        pytest.param(
+            [("200 OK", "regional_memorystore_standard_tier.json"),],
+            False,
+            id="misconfiguredMaxmemory",
+        ),
+        pytest.param(
+            [("200 OK", "regional_memorystore_read_replica_disabled.json"),],
+            False,
+            id="readReplicaDisabled",
+        ),
+        pytest.param(
+            [("200 OK", "regional_memorystore_happy.json"),], True, id="happy",
+        ),
+    ],
+)
+def test_verify_memorystore(
+    setup_mock_server, responses, expected_result: bool, request
+):
+    factory, tracker = setup_mock_server
+    cloud_resources = generate_gcp_cloud_mock_resources()
+
+    if request.node.callspec.id != "vpcMismatch":
+        # This is the vpc id used in frontend/cli/tests/gcp_responses/regional_filestore.json
+        cloud_resources.gcp_vpc_id = "anyscale-vpn-cloud"
+
+    tracker.reset({".*": responses})
+    assert (
+        verify_memorystore(factory, cloud_resources, _gcp_logger(), strict=True,)
+        == expected_result
+    )
+
+
 @patch.multiple(
     "anyscale.gcp_verification", ANYSCALE_HOST="https://console.anyscale.com"
 )
