@@ -6,6 +6,10 @@ import time
 from typing import Any, Dict
 from unittest.mock import Mock, patch
 
+import boto3
+from botocore.exceptions import ClientError
+from click import ClickException
+from moto import mock_ec2
 from packaging import version
 import pytest
 
@@ -14,9 +18,12 @@ from anyscale.aws_iam_policies import (
     DEFAULT_RAY_IAM_ASSUME_ROLE_POLICY,
     get_anyscale_aws_iam_assume_role_policy,
 )
+from anyscale.client.openapi_client.models import CloudAnalyticsEventCloudResource
 from anyscale.conf import MINIMUM_RAY_VERSION
 from anyscale.util import (
     _check_python_version,
+    _get_memorydb_cluster_config,
+    _get_subnet,
     _ray_version_major_minor,
     _update_external_ids_for_policy,
     contains_control_plane_role,
@@ -33,6 +40,7 @@ from anyscale.util import (
     updating_printer,
     verify_data_plane_role_assume_role_policy,
 )
+from anyscale.utils.cloud_utils import CloudSetupError
 from anyscale.utils.name_utils import gen_valid_name
 
 
@@ -285,7 +293,7 @@ def test_prepare_cloudformation_template():
 
     with patch("anyscale.util.get_availability_zones", new=Mock(return_value=mock_azs)):
         cfn_template = prepare_cloudformation_template(
-            mock_region, mock_cfn_stack_name, mock_cloud_id
+            mock_region, mock_cfn_stack_name, mock_cloud_id, True
         )
 
     # If you make some changes to the prepare_cloudformation_template(), update the
@@ -672,3 +680,105 @@ def test_gen_valid_name():
 
     assert re.match(NAME_VALIDATION_REGEX_PATTERN, name_without_prefiex) is not None
     assert re.match(NAME_VALIDATION_REGEX_PATTERN, name_with_prefiex) is not None
+
+
+@pytest.mark.parametrize(
+    ("describe_cluster_response", "raise_exception",),
+    [
+        pytest.param({}, True),
+        pytest.param({"Clusters": [{}]}, True),
+        pytest.param(
+            {
+                "Clusters": [
+                    {
+                        "ARN": "mock_cluster_arn",
+                        "ClusterEndpoint": {
+                            "Address": "mock_cluster_endpoint_address",
+                            "Port": 6379,
+                        },
+                    }
+                ]
+            },
+            False,
+        ),
+    ],
+)
+def test_get_memorydb_cluster_config(describe_cluster_response, raise_exception):
+
+    mock_memorydb_cluster_id = "mock_memorydb_cluster_id"
+    mock_region = "mock_region"
+    mock_memorydb_client = Mock(
+        describe_clusters=Mock(return_value=describe_cluster_response)
+    )
+    mock_boto3 = Mock(client=Mock(return_value=mock_memorydb_client))
+
+    with patch("anyscale.util.boto3", new=mock_boto3):
+        if raise_exception:
+            with pytest.raises(ClickException) as exc_info:
+                _get_memorydb_cluster_config(
+                    mock_memorydb_cluster_id, mock_region, Mock()
+                )
+            assert "does not exist." in str(exc_info.value)
+        else:
+            cluster_config = _get_memorydb_cluster_config(
+                mock_memorydb_cluster_id, mock_region, Mock()
+            )
+            assert cluster_config.id == "mock_cluster_arn"
+            assert cluster_config.endpoint == "mock_cluster_endpoint_address:6379"
+
+
+@pytest.mark.parametrize(
+    ("cloud_region", "subnet_region", "success"),
+    [
+        pytest.param("us-east-1", "us-east-1", True),
+        pytest.param("us-east-1", "us-east-2", False),
+    ],
+)
+@mock_ec2
+def test_get_subnet(cloud_region, subnet_region, success):
+    ec2 = boto3.client("ec2", region_name=subnet_region)
+    vpc = ec2.create_vpc(CidrBlock="10.0.0.0/16")
+    subnet = ec2.create_subnet(
+        VpcId=vpc["Vpc"]["VpcId"],
+        CidrBlock="10.0.0.0/24",
+        AvailabilityZone=f"{subnet_region}a",
+    )
+    subnet_arn = subnet["Subnet"]["SubnetArn"]
+
+    if not success:
+        mock_subnet = Mock(
+            return_value=Mock(
+                load=Mock(
+                    side_effect=ClientError(
+                        {
+                            "Error": {
+                                "Code": "InvalidSubnetID.NotFound",
+                                "Message": "Not Found",
+                            }
+                        },
+                        "describe_subnets",
+                    )
+                )
+            )
+        )
+
+    else:
+        mock_subnet = Mock()
+
+    mock_ec2_resource = Mock(Subnet=mock_subnet)
+
+    mock_logger = Mock()
+
+    with patch("anyscale.util._resource", new=Mock(return_value=mock_ec2_resource)):
+        if success:
+            _get_subnet(subnet_arn, cloud_region, mock_logger)
+        else:
+            with pytest.raises(ClickException) as exc_info:
+                _get_subnet(subnet_arn, cloud_region, mock_logger)
+            assert "does not exist. Please make sur" in str(exc_info.value)
+            mock_logger.log_resource_error.assert_called_with(
+                CloudAnalyticsEventCloudResource.AWS_SUBNET,
+                CloudSetupError.RESOURCE_NOT_FOUND,
+            )
+
+    mock_subnet.assert_called_once_with(subnet_arn)
